Attribute VB_Name = "SRIMfit"
'/*======================================================================*/
'/* Source:     SRIMfit.bas
'/* Description:SRIM"Stopping/Range table" interpolate fitting functions.
'/* Coded:
'/*  Ver.3.12 18.10/11 Ay mod Eng.Help, Application.PathSeparator
'/*     mod srMySRwb_open() for changing WB
'/*     add srMcrWBdir(), MySRwbFnNow, MySRwbDirNow
'/*  Ver.3.11 18.10/01 Ay mod for MacOS,  DirSep="\" -> ":".
'/*  Ver.3.10 18.01/04 Ay mod AutoCalc=OFF.
'/*  Ver.3.00 17.04/05 Ay use external data file as MySRwsNow
'/*       then changed sr_*(WS) ; remove WS param
'/*     add sr*_eq_*() func.
'/*     set ts_errE, ts_errV in sr_PrivFunc()
'/*     sr_PublicFunc return val -> variant ; CVErr(xlErrNA) if error
'/*  Ver.2.10 16.11/05 Ay mod AddIn module
'/*  Ver.2.00 16.09/21 Ay mod WS. --> ThisWorkbook.WS.
'/*  Ver.1.00 14.11/28 A.Yoshida(RIKEN) build.
'/*======================================================================*/

' notation of Func. name
'   sr*() : Public,  sr_*() : Private common, sr__*() : Private local use only

'                       ==== Parameter, Return & Units ====
' parameter notation
'  WS   as String       WorkSheet name of SRIMfit data table
'  Ei   as Double       Energy [MeV/u]
'  Ri   as Double       Range , Straggling [um]
'  Uid  as Integer      dE/dX Unit ID number 0..8 (see SRIMfit data table)
'  Th   as Double       Thickness as Tum [micro-m], Tmm [milli-m]
' return value type
'  I: integer, D: double, S: string, B: boolean
'  V: variant (including Excel error code)
'    CVErr()    value  Cell display
'    xlErrValue 2015   #VALUE!    (Wrong type VALUE)
'    xlErrNum   2036   #NUM!       Invalid NUMber as parameter
'    xlErrNA@@2042   #N/A        return value Not Available

'                       ==== Public Functions =====
'       Macro I/F
'  Sub srMySRwb_open(fn)    Open(ReadOnly) Users SRIM workbook file
'  Sub srMySRwb_close()     Close Users SRIM workbook file
'S= srMcrVer()              SRIMfit ver
'S= srMcrPath()             SRIMfit installed path
'S= srMcrWBname()           Current SRIM workbook file name
'S= srMcrWBdir()            Current SRIM workbook file directory path
'I= srMcrWScount()          Number of the Users SRIM worksheet
'V= srMcrWSlist()           Name list array of the Users SRIM worksheet
'  Sub srMcr_WSlist("A1")   make a list of Users SRIM worksheet

'       Work sheet properties
'V= srElmNm(       Z )      get Element Name
'S= srInfoVer(     WS)      get Info. SRIM ver
'I= srInfoIonZ(    WS)      get Info. IonZ
'I= srInfoIonA(    WS)      get Info. IonA
'S= srInfoTrgName( WS)      get Info. Target Name
'S= srInfoTrgNameL(WS)      get Info. Target Name in long format
'D= srInfoTrgDens( WS)      get Info. Target Density in [g/cm3]
'D= srInfoTrgDensA(WS)      get Info. Target Density in [atoms/cm3]
'D= srInfoBrgC    (WS)      get Info. Bragg Correction in [%]
'B= srInfoTgIsGas (WS)      get Info. if the Target is Gas, return "Gas" as string
'D= srInfoTrgPtbl (WS)      get Info. if Gas, P[Pa]   assumed in this SRIM data table
'D= srInfoTrgTtbl (WS)      get Info. if Gas, T[degC] assumed in this SRIM data table
'S= srInfoTgCmAtmNm (WS,i)  Target Composition Atomic Name       (i=0,1,2,... Row offset)
'I= srInfoTgCmAtmNo (WS,i)  Target Composition Atomic Number
'D= srInfoTgCmAtmPct(WS,i)  Target Composition Atomic Percentage
'D= srInfoTgCmMasPct(WS,i)  Target Composition Mass   Percentage
'S= srInfoWScorded(WS)      WS corded info

'       Unit conversion
'V= srLETUNm(      Uid )    LET unit name
'V= srLETCnvF(  WS,Uid)     LET conversion factor
'D= srum2mg       (WS,um)   unit conv. [um]     -> [mg/cm2]
'D= srmg2um       (WS,mg)   unit conv. [mg/cm2] -> [um]

'       Table size of this WS   Min/Max value of..
'D= sr{Min|Max}E(     WS)   E     [MeV/u]
'D= sr{Min|Max}Rng(   WS)   Range [um]
'D= sr{Min|Max}StLng( WS)   Straggling Longitudinal [um]
'D= sr{Min|Max}StLtr( WS)   Straggling Lateral      [um]

'       LET:dE/dX table search
'V= srE2LETe(   WS,Ei,Uid)      E [MeV/u] -> LET:dE/dX(Electric)        [Uid unit]
'V= srE2LETn(   WS,Ei,Uid)      E [MeV/u] -> LET:dE/dX(Nuclear)         [Uid unit]
'V= srE2LETt(   WS,Ei,Uid)      E [MeV/u] -> LET:dE/dX(Elec+Nucl:Total) [Uid unit]
'V= srLET{e|n|t}2E( WS,Lt,Uid,Elh)  Elh= +1/-1= search at Ehigh/low side
'               LET:dE/dX{e|n|t} [Uid unit] -> E [MeV/u] at Ehigh/low side
'V= srMaxLET{e|n|t}( WS,Uid)    Max LET:dE/dX{e|n|t}                    [Uid unit]
'D= srMaxLET{e|n|t}2E(WS)       E [MeV/u] at Max LET:dE/dX{e|n|t}

'       Range table search
'V= srE2Rng(    WS,Ei)          E     [MeV/u]  -> Range [um]
'V= srRng2E(    WS,Ri)          Range [um]     -> E     [MeV/u]

'       Straggling table search
'V= srE2StLng(  WS,Ei)          E [MeV/u] -> Straggling Longitudinal [um]
'V= srE2StLtr(  WS,Ei)          E [MeV/u] -> Straggling Lateral      [um]
'V= srStLng2E(  WS,Ri)          Straggling Longitudinal [um] -> E [MeV/u]
'V= srStLtr2E(  WS,Ri)          Straggling Lateral      [um] -> E [MeV/u]

'       Gas target P, degC convertion
'V= srThkStd(   WS,Pa,degC)        Gas: Thick. conv. factor to this SRIM data table

'       Combination Func(1) Enew / Eold
'V= srEnew(     WS,E0,Tum)         Enew E0 [MeV/u] -> Thk[um]         -> E1 [MeV/u]
'V= srEnewGas(  WS,E0,Tmm,Pa,degC) Enew E0 [MeV/u] -> Tmm[mm],Pa,degC -> E1 [MeV/u]
'V= srEold(     WS,E1,Tum)         Eold E0 [MeV/u] <- Thk[um]         <- E1 [MeV/u]
'V= srEoldGas(  WS,E1,Tmm,Pa,degC) Eold E0 [MeV/u] <- Tmm[mm],Pa,degC <- E1 [MeV/u]

'       Combination Func(2) *_eq_*()
'V= srRng_eq_Eold(      WS1,WS2,Eu10 )          Range equivalent Eold
'V= srEnew_eq_Th(       WS1,WS2,Eu10,Tum1 )     Enew  [MeV/u] equiv. Thick
'V= srEnewt_eq_Th(      WS1,WS2,Eu10,Tum1 )     EnewT [MeV]   equiv. Thick
'V= srEnew_eq_ThGas(    WS1,WS2,Eu10,Tmm1,P1,C1,P2,C2 ) for Gas target
'V= srEnewt_eq_ThGas(   WS1,WS2,Eu10,Tmm1,P1,C1,P2,C2 ) Pa1,degC1,Pa2,degC2
'V= srEnew_eq_Eold(     WS1,WS2,Eu10,Tum1 )     Enew  [MeV/u] equiv. Eold [MeV/u]
'V= srEnewt_eq_Eold(    WS1,WS2,Eu10,Tum1 )     EnewT [MeV]   equiv. Eold
'V= srEnew_eq_EoldGas(  WS1,WS2,Eu10,Tmm1,P1,C1,P2,C2 ) for Gas target
'V= srEnewt_eq_EoldGas( WS1,WS2,Eu10,Tmm1,P1,C1,P2,C2 )
'V= srDEu_eq_Th(        WS1,WS2,Eu10,Tum1 )     dEu   [MeV/u] equiv. Thick
'V= srDEt_eq_Th(        WS1,WS2,Eu10,Tum1 )     dEt   [MeV]   equiv. Thick
'V= srDEu_eq_ThGas(     WS1,WS2,Eu10,Tmm1,P1,C1,P2,C2 ) for Gas target
'V= srDEt_eq_ThGas(     WS1,WS2,Eu10,Tmm1,P1,C1,P2,C2 )

'       Combination Func(3) *_eq_*() which need E-Rng table search
'V= srDEu_eq_Eold(      WS1,WS2,Eu10,Tum1,Ehl ) dEu   [MeV/u] equiv. Eold
'V= srDEt_eq_Eold(      WS1,WS2,Eu10,Tum1,Ehl ) dEt   [MeV]   equiv. Eold
'V= srDEu_eq_EoldGas(   WS1,WS2,Eu10,Tmm1,Ehl,P1,C1,P2,C2 ) for Gas target
'V= srDEt_eq_EoldGas(   WS1,WS2,Eu10,Tmm1,Ehl,P1,C1,P2,C2 )

'                       ==== Public Func. for Debug use =====

'I= srIxEmin(   WS)             start Ix number of E vs Val table
'I= srIxEmax(   WS)             end   Ix number of E vs Val table
'V= srE2Ix(     WS,Ei)          E  -> nearest Ix number
'V= srVal2Ix(   WS,Vc,Clm,Ehl)  Val-> nearest Ix number
'V= srIx2Val(   WS,ixE,Clm)     ix -> Val(Clm)
'V= srIx2ValAry(WS,ix)          ix -> Val(Clm E,SPe,SPn,SPt,Rng,StLng,StLtr)

'                       ==== Private Functions =====
'                       sr_SetWS() should be done before calling
'       Worksheet change CTRL
' Sub sr_SetWS(WSnew)       set MySRwbNow, etc.

'       E(ix) vs. Val(ix) Table search
'D= sr_ixE2Vip(ixE,Clm,Ei)  ixE, Ei -> Val(Ei,Clm) interpolation
'D= sr_ixE2Eip(ixE,Clm,Vc)  ixE,Clm,Vc -> Ei       interpolation
'I= sr_E2Ix(Ei)             Ei -> ixE      search
'D= sr_E2Val(Ei,Clm)        Ei -> Val(Clm) search
'                           Find Vc <= Val(Clm) -> ixE
'I= sr_Val2Ix(Vc,Clm,ixMin,ixMax,Odr)   Odr= +1/-1= Ascending/Descending
'D= sr_Val2E(Vc,Clm)        Vc of Val(Clm) -> E
'I= sr_FpkIx(Clm,ixMin,ixMax,Odr)       Odr= +1/-1= Ascending/Descending
'                           Find Peak Val(Clm) ->ixE
'       Dubug use(1) Table search Func.
' Sub sr_tsWkAdmp(), sr_tiWkAdmp(), sr_enWkAdmp()

'D= sr_E2LET(Ei,Uid,Clm)    E -> LET(Clm) [Uid unit]
'                           LET(Clm)[MeV/(mg/cm2)] -> E
'D= sr_LET2E(Lc,Uid,Clm,Ehl)            Elh= +1/-1= search at Ehigh/low side
'D= sr_MaxLET(Clm)          find max LET(Clm)
'D= sr_MaxLET2E(Clm)        find max LET(Clm) -> E
'D= sr_E2Rng(Ei)            E -> Rng
'D= sr_Rng2E(Ri)            Rng -> E

'       Gas target P, degC convertion
'B= sr_Gas_eq_Gas(WS1,WS2,Pa1,degC1,&Kg1,Pa2,degC2,&Kg2)
'                           Check both WS are Gas and Calc Kg conv. factor

'       Combination Func(1) Enew / Eold
'D= sr_E2Enew(    E0,Tum)   Enew E0      -> Th  -> E1
'D= sr_ixE2Enew(ixE0,Tum)   Enew E0(ixE) -> Th  -> E1
'D= sr_E2Eold(    E1,Tum)   Eold E0      <- Thk <- E1

'       Combination Func(2) *_eq_*()
' Sub sr_eqWkAclr(), sr_eqsWkAclr()     _eq*_ Work Area Clear
' Sub sr_eqWkAdmp(), sr_eqsWkAdmp()     _eq*_ Work Area Clear

'       Combination Func(3) *_eq_*() which need E-Rng table search
'B= sr_dEfpk(   ES,Th,Odr )         Find dEu peak
'I= sr_dE2ixE(  ES,Th,dEG,Ehl)      Search dEu and return ixE
'D= sr_dEip(    Th,dEG,ixE0,ixE1 )  E vs dE interpolation
'       for DBG ExcelSheet use only <need #Const DBG = True>
'D= srdEfpk(    WS,ES,Th,Odr)           Find dEu peak & return dE
'D= srdEfpkE(   WS,ES,Th,Odr)                         & return E0
'I= srdE2ixE(   WS,ES,Th,dEG,Ehl)       Search dEu and return ixE
'D= srdEip(     WS,Th,dEG,ixE0,ixE1 )   E vs dE interpolation

'/*======================================================================*/
Option Explicit
Option Base 0

' Const DBG = True    ' Debug mode switch    for sedE*()

' Const DirSep = "\"  ' for Win-OS
' Const DirSep = ":"  ' for Mac-OS + Excel2011
' Const DirSep = "/"  ' for Mac-OS + Excel2016, for Win-OS + Excel2016
' Const DirSep = Application.PathSeparator
'    <- can NOT def. as Const, but def. in srMySRwb_open()

'       ====================================================
'           Const, WorkArea Variable def.
'       ====================================================

'/*----------------------------------------------------------------------*/
'/* Const.Def.      Work Sheet Structure
'/*----------------------------------------------------------------------*/
Private Const ClmInfo = 3   ' Work sheet information
Private Const ClmE = 4      ' Energy [MeV/u]
Private Const ClmSPe = 5    ' dE/dx Electric [MeV/(mg/cm2)]
Private Const ClmSPn = 6    ' dE/dx Nuclear  [MeV/(mg/cm2)]
Private Const ClmSPt = 7    ' dE/dx Nuclear  [MeV/(mg/cm2)]
Private Const ClmRng = 10   ' Range  [um]
Private Const ClmSPfct = 11 ' dE/dx unit conversion factors
Private Const ClmStLng = 13 ' Straggling Longitudinal [um]
Private Const ClmStLtr = 16 ' Straggling Lateral      [um]
Private Const ClmTgGas = 6  ' Target is Gas ?
Private Const ClmTgAnm = 6  ' Target Atomic Name
Private Const ClmTgAno = 7  ' Target Atomic Number
Private Const ClmTgAP = 8   ' Target Atomic %My
Private Const ClmTgMP = 9   ' Target Mass   %
Private Const ClmCrded = 16 ' Corded info
Private Const RowVer = 3    ' SRIM ver
Private Const RowIonZ = 4   ' Ion Z
Private Const RowIonA = 5   ' Ion A
Private Const RowTgNS = 6   ' Target name Short
Private Const RowTgNL = 7   ' Target name Long
Private Const RowTgDG = 8   ' Target density in [g/cm3]
Private Const RowTgDA = 9   ' Target density in [atoms/cm3]
Private Const RowBrgC = 10  ' Bragg Correction [%]
Private Const RowEmin = 12  ' row # of Emin
Private Const RowEmax = 13  ' row # of Emax
Private Const RowPtbl = 14  ' if Gas, P[Pa]   asuumed in this SRIM data table
Private Const RowTtbl = 15  ' if Gas, T[degC] asuumed in this SRIM data table
Private Const RowSPfct = 6  ' dE/dx unit = 1 .. 8
Private Const RowTgGas = 3  ' Target is Gas ?
Private Const RowTgCmp = 6  ' Target Composition
Private Const RowCrded = 6  ' Corded info

'/*----------------------------------------------------------------------*/
'/* WkArea Def.     Work Book const & variables
'/*----------------------------------------------------------------------*/
Private Const SRIMfitVer = "SRIMfit v.3.12" ' Version No. of this macro module
Private Const MySRwbFnDfa = "MySRIMwb.xlsx" ' Default SRIM work book file name

Private MySRwbFnNow As String               ' Current SRIM work book file name
Private MySRwbDirNow As String              '  it's directory path
Private MySRwbNow As Workbook               ' Current Workbook  pointer
Private MySRwsNow As Worksheet              ' Current Worksheet pointer

'/*----------------------------------------------------------------------*/
'/* WkArea Def.     Work Sheet Variables
'/*----------------------------------------------------------------------*/
Dim WSnow As String                         ' Current Work Sheet name
Dim IxEmin As Integer, IxEmax As Integer    ' Etable row number range
Dim Emin As Double, Emax As Double          ' min, max of E [MeV/u]
Dim Rmin As Double, Rmax As Double          '          of Range [um]
Dim StLngmin As Double, StLngmax As Double  '          of Straggling Longitudinal [um]
Dim StLtrmin As Double, StLtrmax As Double  '          of Straggling Lateral      [um]
Dim SPfct(8) As Double                      ' Stopping Pw conversion factor

'/*----------------------------------------------------------------------*/
'/* WkArea Def.     Table search result
'/*  for ..             ts_ixE  ts_errE ts_errV  (): set by func call
'/* sr_E2Ix()            Set    ClrSet   ---
'/* sr_E2Val()          (set)   (set)    ---    call sr_E2Ix()
'/* sr__Val2Ix{AO|DO}()  Set     ---    ClrSet
'/* sr_Val2Ix()         (set)    ---    (set)   call sr__Val2Ix{AO|DO}()
'/* sr_Val2E()          (set)    ---    (set)   call sr_Val2Ix()
'/* sr__FpkIx{Fw|Bw}()   Set     Clr     ---
'/* sr_FpkIx()          (set)    ---     ---    call sr__FpkIx{Fw|Bw}()
'/* sr__E2LET()         (set)   (set)           call sr_E2Val()
'/* sr_LET2E()           Set     ---    ClrSet  call sr_FpkIx(), sr_Val2Ix()
'/* sr_MaxLET()         (set)    ---     ---    call sr_FpkIx()
'/* sr_MaxLET2E()       (set)    ---     ---    call sr_FpkIx()
'/* sr_E2Rng()          (set)   (set)    ---    call sr_E2Val()
'/* sr_Rng2E()          (set)    ---    (set)   call sr_Val2E()
'/*----------------------------------------------------------------------*/
Dim ts_ixE  As Integer  ' table search result; found ixE, if Err Ix
Dim ts_errE As Boolean  ' ixE search OVF/UDF error
Dim ts_errV As Boolean  ' ixV search OVF/UDF error

'/*----------------------------------------------------------------------*/
'/* WkArea Def.     Table interpolation calc
'/* sr_ixE2Vip(), _ixE2Eip() set the following var.
'/*----------------------------------------------------------------------*/
Dim ti_dE  As Double    ' = .Cells(ixE, ClmE) - .Cells(ixE - 1, ClmE)
Dim ti_dV  As Double    ' = .Cells(ixE, Clm ) - .Cells(ixE - 1, Clm )
Dim ti_Slp As Double    ' = Slope       = dE/dV or dV/dE
Dim ti_E0  As Double    ' = E home pos. = .Cells(ixE, ClmE)
Dim ti_V0  As Double    ' = V home pos. = .Cells(ixE, Clm )
Dim ti_Dif As Double    ' = Difference  = (Ei - .Cells(ixE, ClmE))
                        '              or (Vc - .Cells(ixE, Clm ))

'/*----------------------------------------------------------------------*/
'/* WkArea Def.     Enew calc result of following func.
'/*     sr_E2Enew(), _ixE2Enew(), _E2Eold(), _ixE2dE()
'/*----------------------------------------------------------------------*/
Dim en_E0 As Double, en_R0 As Double    ' E, Rng before
Dim en_E1 As Double, en_R1 As Double    ' E, Rng after
Dim en_dE As Double                     ' dE=E0-E1
Dim en_ixE0 As Integer                  ' table search result of ixE @ E0
Dim en_ixE1 As Integer                  ' table search result of ixE @ E1

'/*----------------------------------------------------------------------*/
'/* WkArea Def.     EQ calc result (1)
'/*     sr_*_eq_*() set values following
'/*----------------------------------------------------------------------*/
                                    ' for WS1
Dim eq_Th1   As Double, eq_A1   As Double   ' Th, A
Dim eq_E10   As Double, eq_R10  As Double   ' E, Rng before
Dim eq_E11   As Double, eq_R11  As Double   ' E, Rng after
Dim eq_dEu1  As Double, eq_dEt1 As Double   ' dEu=Eu0-Eu1, dEt=dEu*A
Dim eq_Et11  As Double                      ' Et=E1*A1
Dim eq_ixE10 As Integer, eq_ixE11 As Integer ' table search result of ixE @ Eu0,Eu1
                                   ' for WS2
Dim eq_Th2   As Double, eq_A2   As Double
Dim eq_E20   As Double, eq_R20  As Double
Dim eq_E21   As Double, eq_R21  As Double
Dim eq_dEu2  As Double, eq_dEt2 As Double
Dim eq_Et21  As Double, eq_rV As Double
Dim eq_ixE20 As Integer, eq_ixE21 As Integer
Dim eq_SwEt  As Boolean                     ' Et mode switch

'/*----------------------------------------------------------------------*/
'/* WkArea Def.     EQ calc result (2) E_dE search
'/*     sr__DE_eq_Eold() set values following
'/*----------------------------------------------------------------------*/
Dim eqs_Estop As Double, eqs_ixEstop As Integer
Dim eqs_dEpk As Double, eqs_ixdEpk As Integer, eqs_EdEpk As Double
Dim eqs_ErrdEfpk As Integer
Dim eqs_dEu2max As Double, eqs_ixdEu2max As Integer, eqs_EdEu2max As Double
Dim eqs_dEt2max As Double
Dim eqs_dEuG  As Double, eqs_ixdEuG As Integer, eqs_EdEuG As Double
Dim eqs_ErrdE2ixE As Integer
Dim eqsi_E0 As Double, eqsi_ixE0 As Integer, eqsi_dE0 As Double
Dim eqsi_E1 As Double, eqsi_ixE1 As Integer, eqsi_dE1 As Double
Dim eqsi_Slp As Double, eqsi_Edf As Double, eqsi_dEdf As Double
Dim eqsi_dEG As Double, eqsi_EG As Double

'       ====================================================
'           Users SRIM workbook handling
'       ====================================================

'/*----------------------------------------------------------------------*/
'/* (Public Proc.) Open MySRIMwb        call from ThisWorkbook calss module
'/*     cf.) MySRwb should be placed in the same directory of this SRIMfit
'/*----------------------------------------------------------------------*/
Public Sub srMySRwb_open(ByVal MyFn As String)
Dim fn As String
Dim RetWB As Workbook                       ' return to this WkBk
Dim WScnt As Integer, i As Integer
Dim DirSep As String
Dim oMySRwbDirNow As String, oMySRwbFnNow As String

'MsgBox "SRIMfit: MySRwb_open() enter"
'Stop
    oMySRwbDirNow = MySRwbDirNow
    oMySRwbFnNow = MySRwbFnNow
    DirSep = Application.PathSeparator
    If (MyFn = "") Then                 ' open default WB
        MySRwbDirNow = ThisWorkbook.Path    ' assume placed in same dir
        fn = MySRwbDirNow & DirSep & MySRwbFnDfa
    Else                                ' open MyFn specified WB
        fn = MyFn
    End If
    MySRwbFnNow = dir(fn)               ' dir() returns file name only
    If MySRwbFnNow = "" Then            ' check its existance
        MsgBox "srMySRwb_open(): SRIMwb open error: " & vbCrLf & _
        fn & vbCrLf & " is not exist.", _
        vbExclamation
        MySRwbDirNow = oMySRwbDirNow
        MySRwbFnNow = oMySRwbFnNow
        Exit Sub
    End If
    MySRwbDirNow = Replace(fn, MySRwbFnNow, "") ' get dir path only
    
    Set RetWB = ActiveWorkbook
    Application.ScreenUpdating = False
    srMySRwb_close                      ' close current WB
    
    Set MySRwbNow = Workbooks.Open( _
        Filename:=fn, _
        UpdateLinks:=False, _
        ReadOnly:=True)
    
    MySRwbNow.Saved = True
    ActiveWindow.Visible = False        ' hide MySRwb window
''  RetWB.Activate                      ' re-activate this WkBk
    
    WScnt = MySRwbNow.Worksheets.Count  ' AutoCalc=OFF for all sheets
    For i = 1 To WScnt Step 1           ' in MySRwbNow
        MySRwbNow.Sheets(i).EnableCalculation = False
    Next i
        
    Application.ScreenUpdating = True
End Sub

'/*----------------------------------------------------------------------*/
'/* (Public Proc.) Close MySRIMwb       call from ThisWorkbook calss module
'/*----------------------------------------------------------------------*/
Public Sub srMySRwb_close()

' MsgBox "SRIMfit: MySRwb_close() enter"
    If Not (MySRwbNow Is Nothing) Then
        MySRwbNow.Close SaveChanges:=False
    End If
    Set MySRwbNow = Nothing
End Sub

'       ====================================================
'           Current Worksheet change CTRL
'       ====================================================

'/*----------------------------------------------------------------------*/
'/* (Priv.Proc.)    Setup WorkSheet Param.
'/*----------------------------------------------------------------------*/
Private Sub sr_SetWS(ByVal WSnew As String)
Dim i As Integer
    If (WSnew <> WSnow) Then
        Set MySRwsNow = MySRwbNow.Worksheets(WSnew)
        With MySRwsNow
            IxEmin = .Cells(RowEmin, ClmInfo)
            IxEmax = .Cells(RowEmax, ClmInfo)
            Emin = .Cells(IxEmin, ClmE)
            Emax = .Cells(IxEmax, ClmE)
            Rmin = .Cells(IxEmin, ClmRng)
            Rmax = .Cells(IxEmax, ClmRng)
            StLngmin = .Cells(IxEmin, ClmStLng)
            StLngmax = .Cells(IxEmax, ClmStLng)
            StLtrmin = .Cells(IxEmin, ClmStLtr)
            StLtrmax = .Cells(IxEmax, ClmStLtr)
            For i = 0 To 7 Step 1
              SPfct(i + 1) = .Cells(RowSPfct + i, ClmSPfct)
            Next i
              SPfct(0) = SPfct(5)
        End With
        ts_errE = False
        ts_errV = False
        WSnow = WSnew
    End If
End Sub

'       ====================================================
'           Macro information
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.)  get Macro Info. SRIMfit ver
'/*----------------------------------------------------------------------*/
Public Function srMcrVer() As String
Attribute srMcrVer.VB_Description = "̊֐́ASRIMfit versionԍ @Ԃ܂B"
Attribute srMcrVer.VB_ProcData.VB_Invoke_Func = " \n20"
    srMcrVer = SRIMfitVer
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  get Macro Info. SRIMfit installed path
'/*----------------------------------------------------------------------*/
Public Function srMcrPath() As String
Attribute srMcrPath.VB_Description = "̊֐́AMySRwb  CXg[Path@Ԃ܂B"
Attribute srMcrPath.VB_ProcData.VB_Invoke_Func = " \n20"
    srMcrPath = ThisWorkbook.Path
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  get Macro Info. Users SRIM workbook file name
'/*----------------------------------------------------------------------*/
Public Function srMcrWBname() As String
Attribute srMcrWBname.VB_Description = "̊֐́A݂ MySRwb  File Name @Ԃ܂B"
Attribute srMcrWBname.VB_ProcData.VB_Invoke_Func = " \n20"
    srMcrWBname = MySRwbFnNow
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  get Macro Info. Users SRIM workbook file directory path
'/*----------------------------------------------------------------------*/
Public Function srMcrWBdir() As String
Attribute srMcrWBdir.VB_Description = "̊֐́A݂ MySRwb  Directory path @Ԃ܂B"
Attribute srMcrWBdir.VB_ProcData.VB_Invoke_Func = " \n20"
    srMcrWBdir = MySRwbDirNow
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  get Macro Info. Number of Users SRIM worksheet
'/*----------------------------------------------------------------------*/
Public Function srMcrWScount() As Integer
Attribute srMcrWScount.VB_Description = "̊֐́AMySRwb Ɋ܂܂ Sheet@Ԃ܂B"
Attribute srMcrWScount.VB_ProcData.VB_Invoke_Func = " \n20"
    srMcrWScount = MySRwbNow.Worksheets.Count
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  get Macro Info. List of Users SRIM worksheet
'/* HowTo call this func, if you make a vertical list
'/*  = Transpose( srMcrWSlist() )  and CTRL+SHIFT+Enter
'/*----------------------------------------------------------------------*/
Public Function srMcrWSlist() As String()
Attribute srMcrWSlist.VB_Description = "̊֐́AMySRwb Ɋ܂܂ SSheet 1DimzŕԂ@Ԃ܂B"
Attribute srMcrWSlist.VB_ProcData.VB_Invoke_Func = " \n20"
Dim WSnm() As String
Dim WScnt As Integer
Dim i As Integer
    WScnt = srMcrWScount()
    ReDim WSnm(1 To WScnt)
    For i = 1 To WScnt Step 1
        WSnm(i) = MySRwbNow.Sheets(i).Name
    Next i
    srMcrWSlist = WSnm()
End Function

'/*----------------------------------------------------------------------*/
'/*(Sub.)   get Macro Info. make a list of Users SRIM worksheet
'/* HowTo call this sub
'/*     make a button on your sheet, and describe its macro as following
'/*     eg.) make a list from Cell A10 on your ActiveSheet
'/* Private Sub CommandButton1_Click()
'/*    Call Application.Run("srMcr_WSlist","A10")
'.* End Sub
'/*----------------------------------------------------------------------*/
Public Sub srMcr_WSlist(ByVal sToRngS As String)
Dim ToWS As Worksheet
Dim ToRng As Range
Dim i As Integer
Dim ss As String

    Set ToWS = ActiveSheet
    Set ToRng = Range(sToRngS)
    For i = 1 To srMcrWScount() Step 1
        ToRng.Offset(0, 0).Value = i
        ss = MySRwbNow.Sheets(i).Name
        ToRng.Offset(0, 1).Value = MySRwbNow.Sheets(i).Name
        Set ToRng = ToRng.Offset(1, 0)
    Next i
End Sub

'       ====================================================
'           E(ix) vs. Val(ix) Table search
'       ====================================================
'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  ixE, Ei -> Val(Ei,Clm) interpolation
'/*         between Val(ixE,Clm) and Val(ixE-1,Clm)
'/*----------------------------------------------------------------------*/
Private Function sr_ixE2Vip(ByVal ixE As Integer, ByVal Clm As Integer, _
        ByVal Ei As Double) As Double
            
    With MySRwsNow
        ti_E0 = .Cells(ixE, ClmE)
        ti_V0 = .Cells(ixE, Clm)
        ti_Dif = Ei - ti_E0
        ti_dE = ti_E0 - .Cells(ixE - 1, ClmE)
        ti_dV = ti_V0 - .Cells(ixE - 1, Clm)
        ti_Slp = ti_dV / ti_dE
        sr_ixE2Vip = ti_V0 + ti_Slp * ti_Dif
    End With
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  ixE,Clm,Vc -> Ei interpolation
'/*         between Val(ixE,Clm) and Val(ixE-1,Clm)
'/*----------------------------------------------------------------------*/
Private Function sr_ixE2Eip(ByVal ixE As Integer, ByVal Clm As Integer, _
        ByVal Vc As Double) As Double
            
    With MySRwsNow
        ti_E0 = .Cells(ixE, ClmE)
        ti_V0 = .Cells(ixE, Clm)
        ti_Dif = Vc - ti_V0
        ti_dE = ti_E0 - .Cells(ixE - 1, ClmE)
        ti_dV = ti_V0 - .Cells(ixE - 1, Clm)
        ti_Slp = ti_dE / ti_dV
        sr_ixE2Eip = ti_E0 + ti_Slp * ti_Dif
    End With
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  E -> Index number of E[MeV/u]
'/*   return < 0 & ts_errE : if Ei is out of table
'/*----------------------------------------------------------------------*/
Private Function sr_E2Ix(ByVal Ei As Double) As Integer

Dim ixE As Integer
    ts_errE = False
    ts_ixE = 0
    With MySRwsNow
        Select Case Ei
            Case Is < Emin:
                ts_errE = True
                ixE = -IxEmin
            Case Is > Emax:
                ts_errE = True
                ixE = -IxEmax
            Case Else:
                For ixE = IxEmin To IxEmax Step 1
                    If (Ei <= .Cells(ixE, ClmE)) Then Exit For
                Next ixE
        End Select
        sr_E2Ix = ixE
        ts_ixE = Abs(ixE)
    End With
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  E -> Value in a table of Cells(IxEmin..IxEmax,Clm)
'/*   set ts_errV : if Value is out of table
'/*----------------------------------------------------------------------*/
Private Function sr_E2Val(ByVal Ei As Double, ByVal Clm As Integer) As Double

Dim ixE As Integer
    ixE = sr_E2Ix(Ei)
    With MySRwsNow
        If ixE < 0 Then
            sr_E2Val = .Cells(Abs(ixE), Clm)
        Else
            sr_E2Val = sr_ixE2Vip(ixE, Clm, Ei)
        End If
    End With
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  Value in Clm -> Index number of E[MeV/u]
'/*   Odr =  1: search Ascending  ordered table in Cells(from IxMin to IxMax, Clm)
'/*       = -1: search Descending ordered table
'/*   return < 0 if Vc is out of table
'/*----------------------------------------------------------------------*/
Private Function sr__Val2IxAO(ByVal Vc As Double, ByVal Clm As Integer, _
        ByVal ixMin As Integer, ByVal ixMax As Integer) As Integer

Dim ixE As Integer
    ts_errV = False
    ts_ixE = 0
    With MySRwsNow
        Select Case Vc
        Case Is < .Cells(ixMin, Clm):
            ts_errV = True
            ixE = -ixMin
        Case Is > .Cells(ixMax, Clm):
            ts_errV = True
            ixE = -ixMax
        Case Else:
            For ixE = ixMin To ixMax Step 1
               If (Vc <= .Cells(ixE, Clm)) Then Exit For
            Next ixE
        End Select
        sr__Val2IxAO = ixE
        ts_ixE = Abs(ixE)
    End With
End Function

Private Function sr__Val2IxDO(ByVal Vc As Double, ByVal Clm As Integer, _
        ByVal ixMin As Integer, ByVal ixMax As Integer) As Integer

Dim ixE As Integer
    ts_errV = False
    ts_ixE = 0
    With MySRwsNow
        Select Case Vc
        Case Is > .Cells(ixMin, Clm):
            ts_errV = True
            ixE = -ixMin
        Case Is < .Cells(ixMax, Clm):
            ts_errV = True
            ixE = -ixMax
        Case Else:
            For ixE = ixMin To ixMax Step 1
               If (Vc >= .Cells(ixE, Clm)) Then Exit For
            Next ixE
        End Select
        sr__Val2IxDO = ixE
        ts_ixE = Abs(ixE)
    End With
End Function

Private Function sr_Val2Ix(ByVal Vc As Double, ByVal Clm As Integer, _
        ByVal ixMin As Integer, ByVal ixMax As Integer, _
        ByVal Odr As Integer) As Integer
    
    If (Odr < 0) Then
      sr_Val2Ix = sr__Val2IxDO(Vc, Clm, ixMin, ixMax)
    Else
      sr_Val2Ix = sr__Val2IxAO(Vc, Clm, ixMin, ixMax)
    End If
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  Value -> E in a table of Cells(IxEmin..IxEmax,Clm)
'/*----------------------------------------------------------------------*/
Private Function sr_Val2E(ByVal Vc As Double, ByVal Clm As Integer) As Double

Dim ixE As Integer
    ixE = sr_Val2Ix(Vc, Clm, IxEmin, IxEmax, 1)
    With MySRwsNow
        If ixE < 0 Then
            sr_Val2E = .Cells(Abs(ixE), Clm)
        Else
            sr_Val2E = sr_ixE2Eip(ixE, Clm, Vc)
        End If
    End With
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  Find Peak in Clm -> Index number of E[MeV/u]
'/*   Odr =  1: search Forward in Cells(from IxMin to IxMax, Clm)
'/*       = -1: search Backward              IxMax to IxMin
'/*   return < 0 if Vc is out of table
'/*----------------------------------------------------------------------*/
Private Function sr__FpkIxFw(ByVal Clm As Integer, _
        ByVal ixMin As Integer, ByVal ixMax As Integer) As Integer

Dim ixE As Integer, Vc As Double
    ts_errE = False
    ts_ixE = 0
    With MySRwsNow
        Vc = .Cells(ixMin, Clm)
        For ixE = ixMin + 1 To ixMax - 1 Step 1
            If ((Vc >= .Cells(ixE, Clm)) And (Vc >= .Cells(ixE + 1, Clm))) Then
            '   check two points for safety
                Exit For
            End If
            Vc = .Cells(ixE, Clm)
        Next ixE
        sr__FpkIxFw = ixE - 1
        ts_ixE = ixE - 1
    End With
End Function

Private Function sr__FpkIxBw(ByVal Clm As Integer, _
        ByVal ixMin As Integer, ByVal ixMax As Integer) As Integer

Dim ixE As Integer, Vc As Double
    ts_errE = False
    ts_ixE = 0
    With MySRwsNow
        Vc = .Cells(ixMax, Clm)
        For ixE = ixMax - 1 To ixMin + 1 Step -1
            If ((Vc >= .Cells(ixE, Clm)) And (Vc >= .Cells(ixE - 1, Clm))) Then
            '   check two points for safety
                Exit For
            End If
            Vc = .Cells(ixE, Clm)
        Next ixE
        sr__FpkIxBw = ixE + 1
        ts_ixE = ixE + 1
    End With
End Function

Private Function sr_FpkIx(ByVal Clm As Integer, _
        ByVal ixMin As Integer, ByVal ixMax As Integer, ByVal Odr As Integer) As Integer
    If (Odr < 0) Then
        sr_FpkIx = sr__FpkIxBw(Clm, ixMin, ixMax)
    Else
        sr_FpkIx = sr__FpkIxFw(Clm, ixMin, ixMax)
    End If
End Function

'       ====================================================
'           Private func. for Dubug use(1) Table search
'       ====================================================
'/*----------------------------------------------------------------------*/
'/*(Private Func.)  ts_* Work Area Dump as Debug.Print
'/*----------------------------------------------------------------------*/
Private Sub sr_tsWkAdmp()
    Debug.Print "ts_ixE="; ts_ixE; "_errE="; ts_errE; " _errV="; ts_errV
End Sub

'/*----------------------------------------------------------------------*/
'/*(Private Func.)  ti_* Work Area Dump as Debug.Print
'/*----------------------------------------------------------------------*/
Private Sub sr_tiWkAdmp()
    Debug.Print "ti_E0="; ti_E0; " _V0="; ti_V0; _
                 " _dE="; ti_dE; " _dV="; ti_dV; " _Slp="; ti_Slp
End Sub

'/*----------------------------------------------------------------------*/
'/*(Private Func.)  en_* Work Area Dump as Debug.Print
'/*----------------------------------------------------------------------*/
Private Sub sr_enWkAdmp()
    Debug.Print "en_E0="; en_E0; "@_ixE0="; en_ixE0; " _R0="; en_R0
    Debug.Print "  _E1="; en_E1; "@_ixE1="; en_ixE1; " _R1="; en_R1
    Debug.Print "  _dE="; en_dE
End Sub

'       ====================================================
'           Public Func. for Debug use
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.)  return IxEmin
'/*----------------------------------------------------------------------*/
Public Function srIxEmin(ByVal WS As String) As Integer
    sr_SetWS (WS)
    srIxEmin = IxEmin
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  return IxEmax
'/*----------------------------------------------------------------------*/
Public Function srIxEmax(ByVal WS As String) As Integer
    sr_SetWS (WS)
    srIxEmax = IxEmax
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  E[MeV/u] -> nearest Index number of E vs Val table
'/*----------------------------------------------------------------------*/
Public Function srE2Ix(ByVal WS As String, ByVal Ei As Double) As Variant
Dim ixE As Integer
    sr_SetWS (WS)
    ixE = sr_E2Ix(Ei)
    srE2Ix = IIf(ts_errE, CVErr(xlErrNA), ixE)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Value -> nearest Index number of E vs Val table
'/*     Clm# = 1:E, 2:SPe, 3:SPn, 4:SPt, 5:Rng, 6:StLng, 7:StLtr
'/*----------------------------------------------------------------------*/
Public Function srVal2Ix(ByVal WS As String, ByVal Vc As Double, _
    ByVal Clm As Integer, ByVal Ehl As Integer) As Variant

Dim ixE As Integer, ClmNo As Integer, rV As Variant
    sr_SetWS (WS)
    If ((Clm < 0) Or (Clm > 7)) Then GoTo Err1
    Select Case Clm
      Case Is = 1: ClmNo = ClmE
      Case Is = 2: ClmNo = ClmSPe
      Case Is = 3: ClmNo = ClmSPn
      Case Is = 4: ClmNo = ClmSPt
      Case Is = 5: ClmNo = ClmRng
      Case Is = 6: ClmNo = ClmStLng
      Case Is = 7: ClmNo = ClmStLtr
    End Select
    If (Ehl = -1) Then
        ixE = sr_Val2Ix(Vc, ClmNo, IxEmin, IxEmax, 1)
    Else
        ixE = sr_Val2Ix(Vc, ClmNo, IxEmin, IxEmax, -1)
    End If
    rV = IIf(ts_errV, CVErr(xlErrNA), ixE)
    GoTo ExF
Err1: rV = CVErr(xlErrNA)
ExF:  srVal2Ix = rV
' Debug.Print "srVal2Ix(Vc="; Vc; " ClmNo="; ClmNo; " Ehl="; Ehl; ")= "; rV
' sr_tsWkAdmp
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  ixE -> Value(Clm#)
'/*     Clm# = 1:E, 2:SPe, 3:SPn, 4:SPt, 5:Rng, 6:StLng, 7:StLtr
'/*----------------------------------------------------------------------*/
Public Function srIx2Val(ByVal WS As String, ByVal ixE As Integer, _
    ByVal Clm As Integer) As Variant
 
Dim rV As Variant, ClmNo As Integer
    sr_SetWS (WS)
    If ((ixE < IxEmin) Or (ixE > IxEmax) Or (Clm < 0) Or (Clm > 7)) Then GoTo Err1
    Select Case Clm
      Case Is = 1: ClmNo = ClmE
      Case Is = 2: ClmNo = ClmSPe
      Case Is = 3: ClmNo = ClmSPn
      Case Is = 4: ClmNo = ClmSPt
      Case Is = 5: ClmNo = ClmRng
      Case Is = 6: ClmNo = ClmStLng
      Case Is = 7: ClmNo = ClmStLtr
    End Select
    rV = MySRwsNow.Cells(ixE, ClmNo)
    GoTo ExF
Err1: rV = CVErr(xlErrNA)
ExF:  srIx2Val = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  ixE -> Value(ClmE,ClmSPe,ClmSPn,ClmSPt,ClmRng,ClmStLng,ClmStLtr)
'/*----------------------------------------------------------------------*/
Public Function srIx2ValAry(ByVal WS As String, ByVal ixE As Integer) _
    As Variant
 
Dim V(1 To 7) As Double
    sr_SetWS (WS)
    If ((ixE < IxEmin) Or (ixE > IxEmax)) Then
         srIx2ValAry = CVErr(xlErrNA)
         Exit Function
    End If
    With MySRwsNow
        V(1) = .Cells(ixE, ClmE)
        V(2) = .Cells(ixE, ClmSPe)
        V(3) = .Cells(ixE, ClmSPn)
        V(4) = .Cells(ixE, ClmSPt)
        V(5) = .Cells(ixE, ClmRng)
        V(6) = .Cells(ixE, ClmStLng)
        V(7) = .Cells(ixE, ClmStLtr)
    End With
    srIx2ValAry = V()
End Function

'       ====================================================
'           Work sheet properties
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.)  Element Name
'/*----------------------------------------------------------------------*/
Public Function srElmNm(ByVal Z As Integer) As Variant
Attribute srElmNm.VB_Description = "̊֐́AfL@Ԃ܂B"
Attribute srElmNm.VB_ProcData.VB_Invoke_Func = " \n20"
Static ElmNm As Variant
Static swInitENm As Integer
    If (swInitENm <> -1) Then
        swInitENm = -1
ElmNm = Array("n", "H", "He", _
"Li", "Be", "B", "C", "N", "O", "F", "Ne", _
"Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar", _
"K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", _
            "Ga", "Ge", "As", "Se", "Br", "Kr", _
"Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", _
            "In", "Sn", "Sb", "Te", "I", "Xe", _
"Cs", "Ba", "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu", _
                  "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", _
            "Tl", "Pb", "Bi", "Po", "At", "Rn", _
"Fr", "Ra", "Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No", "Lr", _
                  "Rf", "Db", "Sg", "Bh", "Hs", "Mt", "Ds", "Rg", "Cn", "Nh", "Fl", "Mc", "Lv", "Ts", "Og")
    End If
    srElmNm = IIf(((Z < 1) Or (Z > 118)), CVErr(xlErrNum), ElmNm(Z))
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. SRIM ver
'/*----------------------------------------------------------------------*/
Public Function srInfoVer(ByVal WS As String) As String
Attribute srInfoVer.VB_Description = "̊֐́AWS SRIM versionԍ@Ԃ܂B"
Attribute srInfoVer.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoVer = MySRwsNow.Cells(RowVer, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Ion Z
'/*----------------------------------------------------------------------*/
Public Function srInfoIonZ(ByVal WS As String) As Double
Attribute srInfoIonZ.VB_Description = "̊֐́AWS Beam Z ; fԍ@Ԃ܂B"
Attribute srInfoIonZ.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoIonZ = MySRwsNow.Cells(RowIonZ, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Ion A
'/*----------------------------------------------------------------------*/
Public Function srInfoIonA(ByVal WS As String) As Double
Attribute srInfoIonA.VB_Description = "̊֐́AWS Beam A ; ʐ@Ԃ܂B"
Attribute srInfoIonA.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoIonA = MySRwsNow.Cells(RowIonA, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Target Name in short format
'/*----------------------------------------------------------------------*/
Public Function srInfoTrgName(ByVal WS As String) As String
Attribute srInfoTrgName.VB_Description = "̊֐́AWS Target (Zk`)@Ԃ܂B"
Attribute srInfoTrgName.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTrgName = MySRwsNow.Cells(RowTgNS, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Target Name in long format
'/*----------------------------------------------------------------------*/
Public Function srInfoTrgNameL(ByVal WS As String) As String
Attribute srInfoTrgNameL.VB_Description = "̊֐́AWS Target (ڍז)@Ԃ܂B"
Attribute srInfoTrgNameL.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTrgNameL = MySRwsNow.Cells(RowTgNL, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Target Density in [g/cm3]
'/*----------------------------------------------------------------------*/
Public Function srInfoTrgDens(ByVal WS As String) As Double
Attribute srInfoTrgDens.VB_Description = "̊֐́AWS Target x [g/cm3]@Ԃ܂B"
Attribute srInfoTrgDens.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTrgDens = MySRwsNow.Cells(RowTgDG, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Target Density in [atoms/cm3]
'/*----------------------------------------------------------------------*/
Public Function srInfoTrgDensA(ByVal WS As String) As Double
Attribute srInfoTrgDensA.VB_Description = "̊֐́AWS Target x [atoms/cm3]@Ԃ܂B"
Attribute srInfoTrgDensA.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTrgDensA = MySRwsNow.Cells(RowTgDA, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Bragg Correction in [%]
'/*----------------------------------------------------------------------*/
Public Function srInfoBrgC(ByVal WS As String) As Double
Attribute srInfoBrgC.VB_Description = "̊֐́AWS Bragg Correction in [%]@Ԃ܂B"
Attribute srInfoBrgC.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoBrgC = MySRwsNow.Cells(RowBrgC, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. if Gas, P[Pa]   asuumed in this SRIM data table
'/*----------------------------------------------------------------------*/
Public Function srInfoTrgPtbl(ByVal WS As String) As Double
Attribute srInfoTrgPtbl.VB_Description = "̊֐́AWS(GasTrg)̏ꍇASRIMvZɗpGas [Pa] @Ԃ܂B"
Attribute srInfoTrgPtbl.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTrgPtbl = MySRwsNow.Cells(RowPtbl, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. if Gas, T[degC] asuumed in this SRIM data table
'/*----------------------------------------------------------------------*/
Public Function srInfoTrgTtbl(ByVal WS As String) As Double
Attribute srInfoTrgTtbl.VB_Description = "̊֐́AWS(GasTrg)̏ꍇASRIMvZɗpGasx [] @Ԃ܂B"
Attribute srInfoTrgTtbl.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTrgTtbl = MySRwsNow.Cells(RowTtbl, ClmInfo)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. Target is Gas ?
'/*----------------------------------------------------------------------*/
Public Function srInfoTgIsGas(ByVal WS As String) As Boolean
Attribute srInfoTgIsGas.VB_Description = "̊֐́AWS (GasTrg)̏ꍇ True @Ԃ܂B"
Attribute srInfoTgIsGas.VB_ProcData.VB_Invoke_Func = " \n20"
Dim ss As String
    sr_SetWS (WS)
    ss = MySRwsNow.Cells(RowTgGas, ClmTgGas)
    srInfoTgIsGas = IIf((ss = "Gas"), True, False)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get Info. Trg. Compo. Atomic Name     (i=0,1,2,... Row offset)
'/*----------------------------------------------------------------------*/
Public Function srInfoTgCmAtmNm(ByVal WS As String, ByVal i As Integer) As String
Attribute srInfoTgCmAtmNm.VB_Description = "̊֐́AWS Trgg q@Ԃ܂B"
Attribute srInfoTgCmAtmNm.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTgCmAtmNm = MySRwsNow.Cells(RowTgCmp + i, ClmTgAnm)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get Info. Trg. Compo. Atomic Number   (i=0,1,2,... Row offset)
'/*----------------------------------------------------------------------*/
Public Function srInfoTgCmAtmNo(ByVal WS As String, ByVal i As Integer) As Integer
Attribute srInfoTgCmAtmNo.VB_Description = "̊֐́AWS Trgg qԍ@Ԃ܂B"
Attribute srInfoTgCmAtmNo.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTgCmAtmNo = MySRwsNow.Cells(RowTgCmp + i, ClmTgAno)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get Info. Trg. Compo. Atomic Percentage (i=0,1,2,... Row offset)
'/*----------------------------------------------------------------------*/
Public Function srInfoTgCmAtmPct(ByVal WS As String, ByVal i As Integer) As Double
Attribute srInfoTgCmAtmPct.VB_Description = "̊֐́AWS Trgg Atomic [%]@Ԃ܂B"
Attribute srInfoTgCmAtmPct.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTgCmAtmPct = MySRwsNow.Cells(RowTgCmp + i, ClmTgAP)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get Info. Trg. Compo. Mass Percentage   (i=0,1,2,... Row offset)
'/*----------------------------------------------------------------------*/
Public Function srInfoTgCmMasPct(ByVal WS As String, ByVal i As Integer) As Double
Attribute srInfoTgCmMasPct.VB_Description = "̊֐́AWS Trgg Mass [%]@Ԃ܂B"
Attribute srInfoTgCmMasPct.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoTgCmMasPct = MySRwsNow.Cells(RowTgCmp + i, ClmTgMP)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get WS Info. WS corded info
'/*----------------------------------------------------------------------*/
Public Function srInfoWScorded(ByVal WS As String) As String
Attribute srInfoWScorded.VB_Description = "̊֐́AWS Corded@Ԃ܂B"
Attribute srInfoWScorded.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srInfoWScorded = MySRwsNow.Cells(RowCrded, ClmCrded)
End Function

'       ====================================================
'           Unit conversion
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.) get Info. LET unit name
'/*----------------------------------------------------------------------*/
Public Function srLETUNm(ByVal Uid As Integer) As Variant
Attribute srLETUNm.VB_Description = "̊֐́ALETPʖ@Ԃ܂B"
Attribute srLETUNm.VB_ProcData.VB_Invoke_Func = " \n20"

Static LETUNm As Variant
Static swInitUNm As Integer
    If (swInitUNm <> -1) Then
        swInitUNm = -1
LETUNm = Array("MeV/(mg/cm2)", "eV/A", "keV/um", "MeV/mm", "keV/(ug/cm2)", _
               "MeV/(mg/cm2)", "keV/(mg/cm2)", "eV/1E15 atoms/cm2)", "L.S.S.")
    End If
    If ((Uid < 0) Or (Uid > 8)) Then
        srLETUNm = CVErr(xlErrNum)
        Exit Function
    End If
    srLETUNm = LETUNm(Uid)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) get Info. LET conversion factor
'/*----------------------------------------------------------------------*/
Public Function srLETCnvF(ByVal WS As String, ByVal Uid As Integer) As Variant
Attribute srLETCnvF.VB_Description = "̊֐́ALETP ϊW; [Uid=0 MeV/(mg/cm2)] ɕϊ邽߂̌W@Ԃ܂B"
Attribute srLETCnvF.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    If ((Uid < 0) Or (Uid > 8)) Then
        srLETCnvF = CVErr(xlErrNum)
        Exit Function
    End If
    srLETCnvF = SPfct(Uid)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) unit conv. [um] --> [mg/cm2]
'/*----------------------------------------------------------------------*/
Public Function srum2mg(ByVal WS As String, ByVal um As Double) As Double
Attribute srum2mg.VB_Description = "̊֐́ATrg Pʕϊ [m] --> [mg/cm2]@Ԃ܂B"
Attribute srum2mg.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srum2mg = um * srInfoTrgDens(WS) / 10
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) unit conv. [mg/cm2] --> [um]
'/*----------------------------------------------------------------------*/
Public Function srmg2um(ByVal WS As String, ByVal mg As Double) As Double
Attribute srmg2um.VB_Description = "̊֐́ATrg Pʕϊ [mg/cm2] --> [m] @Ԃ܂B"
Attribute srmg2um.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srmg2um = mg / srInfoTrgDens(WS) * 10
End Function

'       ====================================================
'           Worksheet table size
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.) Min / Max E [MeV/u] in this WS
'/*----------------------------------------------------------------------*/
Public Function srMinE(ByVal WS As String) As Double
Attribute srMinE.VB_Description = "̊֐́AWS͈̔ Beam Emin [MeV/u] ; GlM[@Ԃ܂B"
Attribute srMinE.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMinE = Emin
End Function

Public Function srMaxE(ByVal WS As String) As Double
Attribute srMaxE.VB_Description = "̊֐́AWS͈̔ Beam Emax [MeV/u]  ; GlM[@Ԃ܂B"
Attribute srMaxE.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMaxE = Emax
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) Min / Max Range [um] in this WS
'/*----------------------------------------------------------------------*/
Public Function srMinRng(ByVal WS As String) As Double
Attribute srMinRng.VB_Description = "̊֐́AWS͈̔ Range min [m] ; @Ԃ܂B"
Attribute srMinRng.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMinRng = Rmin
End Function

Public Function srMaxRng(ByVal WS As String) As Double
Attribute srMaxRng.VB_Description = "̊֐́AWS͈̔ Range max [m] ; @Ԃ܂B"
Attribute srMaxRng.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMaxRng = Rmax
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) Min / Max Straggling Longitudinal [um] in this WS
'/*----------------------------------------------------------------------*/
Public Function srMinStLng(ByVal WS As String) As Double
Attribute srMinStLng.VB_Description = "̊֐́AWS͈̔  Straggling Longitudinal min [m] ;  [@Ԃ܂B"
Attribute srMinStLng.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMinStLng = StLngmin
End Function

Public Function srMaxStLng(ByVal WS As String) As Double
Attribute srMaxStLng.VB_Description = "̊֐́AWS͈̔  Straggling Longitudinal max [m] ;  [@Ԃ܂B"
Attribute srMaxStLng.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMaxStLng = StLngmax
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.) Min / Max Straggling Lateral [um] in this WS
'/*----------------------------------------------------------------------*/
Public Function srMinStLtr(ByVal WS As String) As Double
Attribute srMinStLtr.VB_Description = "̊֐́AWS͈̔  Straggling Lateral min [m] ; @Ԃ܂B"
Attribute srMinStLtr.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMinStLtr = StLtrmin
End Function

Public Function srMaxStLtr(ByVal WS As String) As Double
Attribute srMaxStLtr.VB_Description = "̊֐́AWS͈̔  Straggling Lateral max [m] ; @Ԃ܂B"
Attribute srMaxStLtr.VB_ProcData.VB_Invoke_Func = " \n20"
    sr_SetWS (WS)
    srMaxStLtr = StLtrmax
End Function

'       ====================================================
'           LET:dE/dX table search
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.)  E [MeV/u] -> LET:dE/dX{e|n|t} in the unit of Uid
'/*----------------------------------------------------------------------*/
Private Function sr__E2LET(ByVal Ei As Double, _
    ByVal Uid As Integer, ByVal Clm As Integer) As Variant

Dim V As Double
    If ((Uid < 0) Or (Uid > 8)) Then GoTo Err1
    V = sr_E2Val(Ei, Clm)
    sr__E2LET = IIf(ts_errE, CVErr(xlErrNA), SPfct(Uid) * V)
    Exit Function
Err1: sr__E2LET = CVErr(xlErrNum)
End Function

Public Function srE2LETe(ByVal WS As String, ByVal Ei As Double, _
    ByVal Uid As Integer) As Variant
Attribute srE2LETe.VB_Description = "̊֐́AWS E -> LETe [UidP] ; Electric Stopping Power @Ԃ܂B"
Attribute srE2LETe.VB_ProcData.VB_Invoke_Func = " \n20"
    
    sr_SetWS (WS)
    srE2LETe = sr__E2LET(Ei, Uid, ClmSPe)
End Function

Public Function srE2LETn(ByVal WS As String, ByVal Ei As Double, _
    ByVal Uid As Integer) As Variant
Attribute srE2LETn.VB_Description = "̊֐́AWS E -> LETn [UidP] ; Nuclear Stopping Power @Ԃ܂B"
Attribute srE2LETn.VB_ProcData.VB_Invoke_Func = " \n20"
    
    sr_SetWS (WS)
    srE2LETn = sr__E2LET(Ei, Uid, ClmSPn)
End Function

Public Function srE2LETt(ByVal WS As String, ByVal Ei As Double, _
    ByVal Uid As Integer) As Variant
Attribute srE2LETt.VB_Description = "̊֐́AWS E -> LETt [UidP] ; Total= Nuclear+Electric Stopping Power @Ԃ܂B"
Attribute srE2LETt.VB_ProcData.VB_Invoke_Func = " \n20"
    
    sr_SetWS (WS)
    srE2LETt = sr__E2LET(Ei, Uid, ClmSPt)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  LET{e|n|t} in the unit of Uid ->  E [MeV/u]
'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  LET{e|n|t} -> Eh, El [MeV/u] higher/lower side
'/*   return = 0 if not found (LET is out of table)
'/*    Lc    LET value (Uid=0; MeV/(mg/cm2)]
'/*    Clm   = ClmSPe|ClmSPn|ClmSPt}
'/*    Ehl   = {1 | -1} ; E higher/lower side switch
'/*----------------------------------------------------------------------*/
Private Function sr_LET2E(ByVal Lc As Double, _
    ByVal Clm As Integer, ByVal Ehl As Integer) As Double

Dim Ipk As Integer, ixE As Integer
Dim Lpk As Double, E As Variant
    
    Ipk = sr_FpkIx(Clm, IxEmin, IxEmax, -1)
    Lpk = MySRwsNow.Cells(Ipk, Clm)
    ts_errV = False
    Select Case Lc
        Case Is > Lpk:
          ts_errV = True
          ts_ixE = 0
          E = 0
        Case Is = Lpk:
          ts_ixE = Ipk
          E = MySRwsNow.Cells(Ipk, ClmE)
        Case Else:
          If (Ehl > 0) Then ' Eh side
            ixE = sr_Val2Ix(Lc, Clm, Ipk, IxEmax, -1) ' Descending order search
          Else              ' El side
            ixE = sr_Val2Ix(Lc, Clm, IxEmin, Ipk, 1)  ' Ascending order serach
          End If
          If ixE < 0 Then
            ts_errV = True              ' ts_IxE <-- sr_Val2Ix()
            E = MySRwsNow.Cells(Abs(ixE), Clm)
          Else
            E = sr_ixE2Eip(ixE, Clm, Lc)
          End If
    End Select
    sr_LET2E = E
End Function

Public Function srLETe2E(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double, ByVal Ehl As Integer) As Variant
Attribute srLETe2E.VB_Description = "̊֐́AWS LETe -> E [MeV/u]@Ԃ܂B"
Attribute srLETe2E.VB_ProcData.VB_Invoke_Func = " \n20"

Dim V As Double, rV As Variant
    If ((Uid < 0) Or (Uid > 8)) Then GoTo Err1
    sr_SetWS (WS)
    V = sr_LET2E(Lt / SPfct(Uid), ClmSPe, Ehl)
    rV = IIf(ts_errV, CVErr(xlErrNA), V)
    GoTo ExF
Err1: rV = CVErr(xlErrNA)
ExF:  srLETe2E = rV
' Debug.Print "srLETe2E(Lt="; Lt; " Uid="; Uid; " Ehl="; Ehl; ")= "; rV
' Call sr_tsWkAdmp: Call sr_tiWkAdmp
End Function

Public Function srLETn2E(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double, ByVal Ehl As Integer) As Variant
Attribute srLETn2E.VB_Description = "̊֐́AWS LETn -> E [MeV/u]@Ԃ܂B"
Attribute srLETn2E.VB_ProcData.VB_Invoke_Func = " \n20"

Dim V As Double, rV As Variant
    If ((Uid < 0) Or (Uid > 8)) Then GoTo Err1
    sr_SetWS (WS)
    V = sr_LET2E(Lt / SPfct(Uid), ClmSPn, Ehl)
    rV = IIf(ts_errV, CVErr(xlErrNA), V)
    GoTo ExF
Err1: rV = CVErr(xlErrNA)
ExF:  srLETn2E = rV
End Function

Public Function srLETt2E(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double, ByVal Ehl As Integer) As Variant
Attribute srLETt2E.VB_Description = "̊֐́AWS LETt -> E [MeV/u]@Ԃ܂B"
Attribute srLETt2E.VB_ProcData.VB_Invoke_Func = " \n20"

Dim V As Double, rV As Variant
    If ((Uid < 0) Or (Uid > 8)) Then GoTo Err1
    sr_SetWS (WS)
    V = sr_LET2E(Lt / SPfct(Uid), ClmSPt, Ehl)
    rV = IIf(ts_errV, CVErr(xlErrNA), V)
    GoTo ExF
Err1: rV = CVErr(xlErrNA)
ExF:  srLETt2E = rV
End Function

'   for lower compatibility
Public Function srLETe2El(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double) As Variant
    srLETe2El = srLETe2E(WS, Lt, Uid, -1)
End Function

Public Function srLETe2Eh(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double) As Variant
    srLETe2Eh = srLETe2E(WS, Lt, Uid, 1)
End Function

Public Function srLETn2El(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double) As Variant
    srLETn2El = srLETn2E(WS, Lt, Uid, -1)
End Function

Public Function srLETn2Eh(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double) As Variant
    srLETn2Eh = srLETn2E(WS, Lt, Uid, 1)
End Function

Public Function srLETt2El(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double) As Variant
    srLETt2El = srLETt2E(WS, Lt, Uid, -1)
End Function

Public Function srLETt2Eh(ByVal WS As String, ByVal Lt As Double, _
    ByVal Uid As Double) As Variant
    srLETt2Eh = srLETt2E(WS, Lt, Uid, 1)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  find Max LET{e|n|t} in the unit of Uid
'/*----------------------------------------------------------------------*/
Private Function sr_MaxLET(ByVal Clm As Integer) As Double
Dim Ipk As Integer
    Ipk = sr_FpkIx(Clm, IxEmin, IxEmax, -1)     'w/o error always
    sr_MaxLET = MySRwsNow.Cells(Ipk, Clm)
End Function

Public Function srMaxLETe(ByVal WS As String, ByVal Uid As Double) As Variant
Attribute srMaxLETe.VB_Description = "̊֐́AWS max LETe [UidP] ; Electric StPw@Ԃ܂B"
Attribute srMaxLETe.VB_ProcData.VB_Invoke_Func = " \n20"

Dim V As Double
    sr_SetWS (WS)
    If ((Uid < 0) Or (Uid > 8)) Then GoTo Err1
    V = sr_MaxLET(ClmSPe)
'   srMaxLETe = IIf((ts_errV), CVErr(xlErrNA), SPfct(Uid) * V) w/o error always
    srMaxLETe = SPfct(Uid) * V
    Exit Function
Err1: srMaxLETe = CVErr(xlErrNum)
End Function

Public Function srMaxLETn(ByVal WS As String, ByVal Uid As Double) As Variant
Attribute srMaxLETn.VB_Description = "̊֐́AWS max LETn [UidP] ; Nuclear StPw@Ԃ܂B"
Attribute srMaxLETn.VB_ProcData.VB_Invoke_Func = " \n20"

Dim V As Double
    sr_SetWS (WS)
    If ((Uid < 0) Or (Uid > 8)) Then GoTo Err1
    V = sr_MaxLET(ClmSPn)
    srMaxLETn = SPfct(Uid) * V
    Exit Function
Err1: srMaxLETn = CVErr(xlErrNum)
End Function

Public Function srMaxLETt(ByVal WS As String, ByVal Uid As Double) As Variant
Attribute srMaxLETt.VB_Description = "̊֐́AWS max LETt [UidP] ; Total= Electric+Nuclear StPw@Ԃ܂B"
Attribute srMaxLETt.VB_ProcData.VB_Invoke_Func = " \n20"

Dim V As Double
    sr_SetWS (WS)
    If ((Uid < 0) Or (Uid > 8)) Then GoTo Err1
    V = sr_MaxLET(ClmSPt)
    srMaxLETt = SPfct(Uid) * V
    Exit Function
Err1: srMaxLETt = CVErr(xlErrNum)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  find E [MeV/u] at Max LET{e|n|t}
'/*----------------------------------------------------------------------*/
Private Function sr_MaxLET2E(ByVal Clm As Integer) As Double
Dim Ipk As Integer
    Ipk = sr_FpkIx(Clm, IxEmin, IxEmax, -1)             'w/o error always
    sr_MaxLET2E = MySRwsNow.Cells(Ipk, ClmE)
End Function

Public Function srMaxLETe2E(ByVal WS As String) As Double
Attribute srMaxLETe2E.VB_Description = "̊֐́AWS E [MeV/u] at max LETe ; Electric StPw@Ԃ܂B"
Attribute srMaxLETe2E.VB_ProcData.VB_Invoke_Func = " \n20"
Dim V As Double
    sr_SetWS (WS)
    V = sr_MaxLET2E(ClmSPe)
'   srMaxLETe2E = IIf((ts_errV), CVErr(xlErrNA), V)     'w/o error always
    srMaxLETe2E = V
End Function

Public Function srMaxLETn2E(ByVal WS As String) As Double
Attribute srMaxLETn2E.VB_Description = "̊֐́AWS E [MeV/u] at max LETn ; Nuclear StPw@Ԃ܂B"
Attribute srMaxLETn2E.VB_ProcData.VB_Invoke_Func = " \n20"
Dim V As Double
    sr_SetWS (WS)
    V = sr_MaxLET2E(ClmSPn)
    srMaxLETn2E = V
End Function

Public Function srMaxLETt2E(ByVal WS As String) As Double
Attribute srMaxLETt2E.VB_Description = "̊֐́AWS E [MeV/u] at max LETt ; Total= Electric+Nuclear StPw@Ԃ܂B"
Attribute srMaxLETt2E.VB_ProcData.VB_Invoke_Func = " \n20"
Dim V As Double
    sr_SetWS (WS)
    V = sr_MaxLET2E(ClmSPt)
    srMaxLETt2E = V
End Function

'       ====================================================
'           Range table search
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.)  E [MeV/u] -> Range [um]
'/*(Priv.Func.) is for Enew/old() calc.
'/*----------------------------------------------------------------------*/
Private Function sr_E2Rng(ByVal Ei As Double) As Double
    sr_E2Rng = sr_E2Val(Ei, ClmRng)
End Function

Public Function srE2Rng(ByVal WS As String, ByVal Ei As Double) As Variant
Attribute srE2Rng.VB_Description = "̊֐́AWS E -> Rng [m] ; @Ԃ܂B"
Attribute srE2Rng.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rV As Variant
    If (Ei < 0) Then GoTo Err1
    If (Ei = 0) Then GoTo Err2
    sr_SetWS (WS)
    rV = sr_E2Rng(Ei)
    If (ts_errE) Then rV = CVErr(xlErrNA)
    GoTo ExF
Err1: rV = CVErr(xlErrNum): GoTo ExF
Err2: rV = 0: GoTo ExF
ExF: srE2Rng = rV
'Debug.Print "srE2Rng(Ei="; Ei; ")= "; rV
'Call sr_tsWkAdmp: Call sr_tiWkAdmp
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Range [um] -> E [MeV/u]
'/*(Priv.Func.) is for Enew/old() calc.
'/*----------------------------------------------------------------------*/
Private Function sr_Rng2E(ByVal Ri As Double) As Double
    sr_Rng2E = sr_Val2E(Ri, ClmRng)
End Function

Public Function srRng2E(ByVal WS As String, ByVal Ri As Double) As Variant
Attribute srRng2E.VB_Description = "̊֐́AWS Rng ;  -> E [MeV/u] @Ԃ܂B"
Attribute srRng2E.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rV As Variant
    If (Ri < 0) Then GoTo Err1
    If (Ri = 0) Then GoTo Err2
    sr_SetWS (WS)
    rV = sr_Rng2E(Ri)
    If (ts_errV) Then rV = CVErr(xlErrNA)
    GoTo ExF
Err1: rV = CVErr(xlErrNum): GoTo ExF
Err2: rV = 0: GoTo ExF
ExF: srRng2E = rV
'Debug.Print "srRng2E(Ri="; Ri; ")= "; rV
'Call sr_tsWkAdmp: Call sr_tiWkAdmp
End Function

'       ====================================================
'           Straggling table search
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.)  E [MeV/u] -> Straggling Longitudinal [um]
'/*----------------------------------------------------------------------*/
Public Function srE2StLng(ByVal WS As String, ByVal Ei As Double) As Variant
Attribute srE2StLng.VB_Description = "̊֐́AWS E -> Straggling Longitudinal [m] ;  [@Ԃ܂B"
Attribute srE2StLng.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rV As Variant
    If (Ei < 0) Then GoTo Err1
    If (Ei = 0) Then GoTo Err2
    sr_SetWS (WS)
    rV = sr_E2Val(Ei, ClmStLng)
    rV = IIf(ts_errE, CVErr(xlErrNA), rV)
    GoTo ExF
Err1: rV = CVErr(xlErrNum): GoTo ExF
Err2: rV = 0: GoTo ExF
ExF: srE2StLng = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  E [MeV/u] -> Straggling Lateral [um]
'/*----------------------------------------------------------------------*/
Public Function srE2StLtr(ByVal WS As String, ByVal Ei As Double) As Variant
Attribute srE2StLtr.VB_Description = "̊֐́AWS E -> Straggling Lateral [m] ; @Ԃ܂B"
Attribute srE2StLtr.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rV As Variant
    If (Ei < 0) Then GoTo Err1
    If (Ei = 0) Then GoTo Err2
    sr_SetWS (WS)
    rV = sr_E2Val(Ei, ClmStLtr)
    rV = IIf(ts_errE, CVErr(xlErrNA), rV)
    GoTo ExF
Err1: rV = CVErr(xlErrNum): GoTo ExF
Err2: rV = 0: GoTo ExF
ExF: srE2StLtr = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Straggling Longitudinal [um] -> E [MeV/u]
'/*----------------------------------------------------------------------*/
Public Function srStLng2E(ByVal WS As String, ByVal Si As Double) As Variant
Attribute srStLng2E.VB_Description = "̊֐́AWS  Straggling Longitudinal;  [ -> E [MeV/u]@Ԃ܂B"
Attribute srStLng2E.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rV As Variant
    If (Si < 0) Then GoTo Err1
    If (Si = 0) Then GoTo Err2
    sr_SetWS (WS)
    rV = sr_Val2E(Si, ClmStLng)
    rV = IIf(ts_errV, CVErr(xlErrNA), rV)
    GoTo ExF
Err1: rV = CVErr(xlErrNum): GoTo ExF
Err2: rV = 0: GoTo ExF
ExF: srStLng2E = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Straggling Lateral [um] -> E [MeV/u]
'/*----------------------------------------------------------------------*/
Public Function srStLtr2E(ByVal WS As String, ByVal Si As Double) As Variant
Attribute srStLtr2E.VB_Description = "̊֐́AWS  Straggling Lateral ;  -> E [MeV/u]@Ԃ܂B"
Attribute srStLtr2E.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rV As Variant
    If (Si < 0) Then GoTo Err1
    If (Si = 0) Then GoTo Err2
    sr_SetWS (WS)
    rV = sr_Val2E(Si, ClmStLtr)
    rV = IIf(ts_errV, CVErr(xlErrNA), rV)
    GoTo ExF
Err1: rV = CVErr(xlErrNum): GoTo ExF
Err2: rV = 0: GoTo ExF
ExF: srStLtr2E = rV
End Function

'       ====================================================
'           Gas target P, degC convertion
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Func.)  Gas: Thick. conv. factor to "Standard P,T" of this SRIM data table
'            Thick@Std [um] or [mg/cm2] = Thick * srThkStd(Pa,degC)
'/*----------------------------------------------------------------------*/
'                                                       Tst      Vst   Pst
' STP : Standard         Temperature and Pressure:  0 C(273.15K),22.7L,1013.25hPa(1atm)
' SATP: Standard Ambient Temperature and Pressure: 25 C(298.15K),24.8L,1000.00hPa(1bar)
'see) TRIM manual http://www.srim.org/SRIM/SRIM%2008.pdf
'     8-12 Stopping Powers for Ions in Gases
'     The stopping of ions in gases is very dependent on the gas pressure.
'     STP (0 C, 760 mm). "TRIM always assumes a gas target is at STP."
'   Pst*Vst/Tst = P*V/T = const         -> Vst  = V  *(P/Pst)*(Tst/T)
'      Dens     = g / V                 -> Dst  = D  *(Pst/P)*(T/Tst)
'      Tk [cm]  = [g/cm2] / Dens[g/cm3] -> Tkst = Tk *(P/Pst)*(Tst/T)
' But.. the [Compund Dictionary] sometimes seemed to be assumed
' different Temperature, Pressure and Density.
' Then, please check and describe the value of "Ptbl" and "Ttbl"
' in the SRIM data table by your own responsibility.
'/*----------------------------------------------------------------------*/
Public Function srThkStd(ByVal WS As String, _
    ByVal Pa As Double, ByVal degC As Double) As Variant
Attribute srThkStd.VB_Description = "̊֐́AGasTrg ́sWĆtւ̕ϊWB@sWĆtƂ́ASRIM WSvZɗpCFsrInfoTrgPtbl()ACFsrInfoTrgTtbl()łB@Ԃ܂B"
Attribute srThkStd.VB_ProcData.VB_Invoke_Func = " \n20"

Dim P0 As Double, T0 As Double, rV As Variant
    If (Not srInfoTgIsGas(WS)) Then GoTo Err1
    If (Pa < 0) Then GoTo Err1
    If (degC < -273.15) Then GoTo Err1
    P0 = srInfoTrgPtbl(WS)      ' read assumed Ptbl[Pa] in this WS
    T0 = srInfoTrgTtbl(WS)      '              Ttbl[degC]
    rV = (Pa / P0) * ((273.15 + T0) / (273.15 + degC))
    GoTo ExF
Err1: rV = CVErr(xlErrNum)
ExF:  srThkStd = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.) common setup for sr*_eq_*Gas() functions
'/*     WS1,WS2         Check both Gas-Target ?
'/*         and return THick Std. conversion factors
'/*     Pa1,degC1,Kg1   Kg1 = srThkStd(WS1, Pa1, degC1)*1000
'/*     Pa2,degC2,Kg2   Kg2 = srThkStd(WS2, Pa2, degC2)*1000
'/*----------------------------------------------------------------------*/
Private Function sr_Gas_eq_Gas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Pa1 As Double, ByVal degC1 As Double, ByRef Kg1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double, ByRef Kg2 As Double) As Boolean

Dim G1 As Boolean, G2 As Boolean, rV As Boolean
    G1 = srInfoTgIsGas(WS1)
    G2 = srInfoTgIsGas(WS2)
    If ((Not G1) Or (Not G2)) Then GoTo Err1
    If ((Pa1 < 0) Or (Pa2 < 0)) Then GoTo Err1
    If ((degC1 < -273.15) Or (degC2 < -273.15)) Then GoTo Err1
    Kg1 = srThkStd(WS1, Pa1, degC1) * 1000  ' [um] -> [mm]
    Kg2 = srThkStd(WS2, Pa2, degC2) * 1000  ' [um] -> [mm]
    rV = True: GoTo ExF
Err1: rV = False
ExF: sr_Gas_eq_Gas = rV
End Function

'       ====================================================
'           Combination Func(1) Enew / Eold
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  E2Enew(E0 Thk) -> return E1
'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  ixE2Enew(ixE0,Thk) -> return E1
'/*----------------------------------------------------------------------*/
'/*              both allow Th<0 as Eold() mode
'/* E1=      when                       en_ixE1=        en_dE=
'/* = -1     R1= R0-Th<0 (stop in Th)   = 0             = E0
'/* =  0     R1= 0 just stopped         = IxEmin        = E0
'/* = -Emax  R1> Rmax (Th<<0)           = -IxEmax       = Emax
'/* >  0     R1= others                 = search res.   = search result
'/*----------------------------------------------------------------------*/
Private Function sr__En(ByVal Th As Double) As Double
    en_R1 = en_R0 - Th
    If (en_R1 < 0) Then
        en_E1 = -1
        en_ixE1 = 0
        en_dE = en_E0
        GoTo ExF
    ElseIf (en_R1 = 0) Then
        en_E1 = 0
        en_ixE1 = IxEmin
    ElseIf (en_R1 > Rmax) Then
        en_E1 = -Emax
        en_ixE1 = -IxEmax
        en_dE = Emax
        GoTo ExF
    Else
        en_E1 = sr_Val2E(en_R1, ClmRng)         ' = sr_Rng2E(en_R1)
        en_ixE1 = ts_ixE
    End If
    en_dE = en_E0 - en_E1
ExF: sr__En = en_E1
End Function

Private Function sr_E2Enew(ByVal E0 As Double, ByVal Th As Double) As Double

Dim rV As Double
    en_E0 = E0
    en_R0 = sr_E2Val(E0, ClmRng)                ' = sr_E2Rng(E0)
    If (ts_errE) Then GoTo Err1
    en_ixE0 = ts_ixE
    rV = sr__En(Th)
    GoTo ExF
Err1: rV = -1                       ' -1 : E0 err   E0 > Emax
      en_ixE0 = 0: en_ixE1 = 0
      en_E1 = 0: en_R1 = 0: en_dE = 0
ExF: sr_E2Enew = rV
End Function

Private Function sr_ixE2Enew(ByVal ixE0 As Integer, ByVal Th As Double) As Double
    
Dim rV As Double
    If ((ixE0 < IxEmin) Or (ixE0 > IxEmax)) Then GoTo Err1
    en_E0 = MySRwsNow.Cells(ixE0, ClmE)
    en_R0 = MySRwsNow.Cells(ixE0, ClmRng)
    en_ixE0 = ixE0
    rV = sr__En(Th)
    GoTo ExF
Err1: rV = -1                       ' -1 : ixE0 err   ixE0<IxEmin | ixE0>IxEmax
      en_ixE0 = 0: en_ixE1 = 0
      en_E1 = 0: en_R1 = 0: en_dE = 0
ExF: sr_ixE2Enew = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Enew E0 [MeV/u] -> Th [um] -> E1 [MeV/u]
'/*----------------------------------------------------------------------*/
'/*(Func.)  Enew in Gas E0 [MeV/u] -> Th [mm],Pa,degC -> E1 [MeV/u]
'/*----------------------------------------------------------------------*/
'/*       E1=           when            * both allow Th<0 as Eold() mode
'/*         = #NUM!     E0 < 0
'/*         = 0         E0 = 0 | E1 = 0 | -Th > Rmax
'/*----------------------------------------------------------------------*/
Public Function srEnew(ByVal WS As String, ByVal E0 As Double, _
    ByVal Tum As Double) As Variant
Attribute srEnew.VB_Description = "̊֐́ATrg Th[m] ʉߌ Beam E [MeV/u] ; Enew@Ԃ܂B"
Attribute srEnew.VB_ProcData.VB_Invoke_Func = " \n20"
    
Dim E1 As Double, rV As Variant
    If (E0 < 0) Then GoTo Err1
    If (E0 = 0) Then GoTo Err2
    sr_SetWS (WS)
    E1 = sr_E2Enew(E0, Tum)
    If (E1 = -1) Then GoTo Err3     ' stopped in Th
    If (E1 = -Emax) Then GoTo Err4      ' rV=       when
    rV = E1: GoTo ExF                   ' >= 0      normal calc | E1=0 just stopped
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   E0 < 0
Err2: rV = 0: GoTo ExF                  ' = 0       E0 = 0
Err3: rV = 0: GoTo ExF                  ' = 0       E1 = -1     as  R0 < Th
Err4: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    E1 = -Emax  as -Th > Rmax
ExF:  srEnew = rV
'Debug.Print "srEnew(E0="; E0; " Tum="; Tum; ")="; rV
'Call sr_enWkAdmp
End Function

Public Function srEnewGas(ByVal WS As String, ByVal E0 As Double, _
    ByVal Tmm As Double, ByVal Pa As Double, ByVal degC As Double) As Variant
Attribute srEnewGas.VB_Description = "̊֐́AGasTrg Th[mm] ʉߌ Beam E [MeV/u] ; Enew@Ԃ܂B"
Attribute srEnewGas.VB_ProcData.VB_Invoke_Func = " \n20"

Dim Tum As Double, E1 As Double, rV As Variant
    If (Not srInfoTgIsGas(WS)) Then GoTo Err1
    If ((E0 < 0)) Then GoTo Err1
    If ((E0 = 0)) Then GoTo Err2
    sr_SetWS (WS)
    Tum = Tmm * 1000 * srThkStd(WS, Pa, degC)
    E1 = sr_E2Enew(E0, Tum)
    If (E1 = -1) Then GoTo Err3     ' stopped in Th
    If (E1 = -Emax) Then GoTo Err4      ' rV=       when
    rV = E1: GoTo ExF                   ' >= 0      normal calc | E1=0 just stopped
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   E0 < 0 | Not Gas
Err2: rV = 0: GoTo ExF                  ' = 0       E0 = 0
Err3: rV = 0: GoTo ExF                  ' = 0       E1 = -1     as  R0 < Th
Err4: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    E1 = -Emax  as -Th > Rmax
ExF:  srEnewGas = rV
'Debug.Print "srEnewGas(E0="; E0; " Tmm="; Tmm; ")="; rV
'Call sr_enWkAdmp
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.)  Eold(E1,Thk) -> return E0
'/*                 should be E1, Th >= 0
'/*   E0=       when                    en_ixE0=        en_dE=
'/* = -1     E1 < 0 | Th < 0            = 0             = 0
'/* = -2     E1 > Emax                  = -IxEmax       = 0
'/* = -Emax  R0 > Rmax                  = -IxEmax       = Emax
'/* =  0     E1 = 0 & Th = 0            = IxEmin        = 0
'/* >  0     R1= others                 = search res.   = search result
'/*----------------------------------------------------------------------*/
Private Function sr_E2Eold(ByVal E1 As Double, ByVal Th As Double) As Double
        
Dim rV As Double
    en_E1 = E1
    If ((en_E1 < 0) Or (Th < 0)) Then
        GoTo Err1
    ElseIf (en_E1 > Emax) Then
        GoTo Err2
    ElseIf (en_E1 = 0) Then
        en_R1 = 0
        en_ixE1 = IxEmin
    Else
        en_R1 = sr_E2Val(E1, ClmRng)            ' = sr_E2Rng(E1)
        en_ixE1 = ts_ixE
    End If
    en_R0 = en_R1 + Th
    If (en_R0 = 0) Then GoTo Err3
    If (en_R0 > Rmax) Then GoTo Err4
    en_E0 = sr_Val2E(en_R0, ClmRng)             ' = sr_Rng2E(R0)
    en_ixE0 = ts_ixE
    en_dE = en_E0 - en_E1
    rV = en_E0
    GoTo ExF
Err1: rV = -1                       ' (E1 < 0) | (Th < 0)
    en_ixE0 = 0: en_ixE1 = 0
    en_E0 = 0: en_R0 = 0: en_R1 = 0: en_dE = 0
    GoTo ExF
Err2: rV = -2                       ' E1 > Emax
    en_ixE0 = -IxEmax: en_ixE1 = IxEmax
    en_E0 = -Emax: en_R0 = -Rmax: en_R1 = -Rmax: en_dE = 0
    GoTo ExF
Err3: rV = 0
    en_ixE0 = IxEmax
    en_E0 = 0: en_R0 = 0: en_dE = 0
    GoTo ExF
Err4: rV = -Emax
    en_ixE0 = -IxEmax
    en_E0 = -Emax: en_R0 = -Rmax: en_dE = Emax
    GoTo ExF
ExF: sr_E2Eold = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Eold E0 [MeV/u] <- Thk [um] <- E1 [MeV/u]
'/*----------------------------------------------------------------------*/
'/*(Func.)  Eold in Gas E0 [MeV/u] <- Thk[mm],Pa,degC <- E1 [MeV/u]
'/*----------------------------------------------------------------------*/
Public Function srEold(ByVal WS As String, ByVal E1 As Double, _
    ByVal Tum As Double) As Variant
Attribute srEold.VB_Description = "̊֐́ATrg Th[m] ʉߑO Beam E [MeV/u] ; Eold@Ԃ܂B"
Attribute srEold.VB_ProcData.VB_Invoke_Func = " \n20"

Dim E0 As Double, rV As Variant
    If (E1 < 0) Then GoTo Err1
    If (Tum < 0) Then GoTo Err1
    sr_SetWS (WS)
    E0 = sr_E2Eold(E1, Tum)
    If (E0 = -2) Then GoTo Err2
    If (E0 = -Emax) Then GoTo Err3      ' rV=       when
    rV = E0: GoTo ExF                   ' >= 0      normal calc
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   E1 < 0 | Tum < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    E1 > Emax
Err3: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    R0 > Rmax
ExF:  srEold = rV
'Debug.Print "srEold(E1="; E1; " Tum="; Tum; ")="; rV
'Call sr_enWkAdmp
End Function

Public Function srEoldGas(ByVal WS As String, ByVal E1 As Double, _
    ByVal Tmm As Double, ByVal Pa As Double, ByVal degC As Double) As Variant
Attribute srEoldGas.VB_Description = "̊֐́AGasTrg Th[mm] ʉߑO Beam E [MeV/u] ; Eold@Ԃ܂B"
Attribute srEoldGas.VB_ProcData.VB_Invoke_Func = " \n20"

Dim E0 As Double, Tum As Double, rV As Variant
    If (Not srInfoTgIsGas(WS)) Then GoTo Err1
    If (E1 < 0) Then GoTo Err1
    If (Tmm < 0) Then GoTo Err1
    sr_SetWS (WS)
    Tum = Tmm * 1000 * srThkStd(WS, Pa, degC)
    E0 = sr_E2Eold(E1, Tum)
    If (E0 = -2) Then GoTo Err2
    If (E0 = -Emax) Then GoTo Err3      ' rV=       when
    rV = E0: GoTo ExF                   ' >= 0      normal calc
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   E1 < 0 | Tum < 0| notGas
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    E1 > Emax
Err3: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    R0 > Rmax
ExF:  srEoldGas = rV
'Debug.Print "srEoldGas(E1="; E1; " Tmm="; Tmm; ")="; rV
'Call sr_enWkAdmp
End Function

'       ====================================================
'           Combination Func(2) *_eq_*()
'       ====================================================
'        notation:  Func.Name = sr**_eq_**() or sr**_eq_**Gas()
'           !Var = parameter,       ?Var = return
'           Et = Total Energy[MeV]  Eu = Energy per unit mass [MeV/u]
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Private Func.)  eq_* Work Area Clear
'/*----------------------------------------------------------------------*/
Private Sub sr_eqWkAclr()
    eq_Th1 = 0: eq_A1 = 0: eq_ixE10 = 0: eq_ixE11 = 0
    eq_E10 = 0: eq_R10 = 0: eq_ixE10 = 0
    eq_E11 = 0: eq_R11 = 0: eq_ixE11 = 0
    eq_dEu1 = 0: eq_dEt1 = 0: eq_Et11 = 0
    eq_Th2 = 0: eq_A2 = 0: eq_ixE20 = 0: eq_ixE21 = 0
    eq_E20 = 0: eq_R20 = 0: eq_ixE20 = 0
    eq_E21 = 0: eq_R21 = 0: eq_ixE21 = 0
    eq_dEu2 = 0: eq_dEt2 = 0: eq_Et21 = 0: eq_SwEt = 0: eq_rV = 0
End Sub

'/*----------------------------------------------------------------------*/
'/*(Private Func.)  eqs_* Work Area Clear
'/*----------------------------------------------------------------------*/
Private Sub sr_eqsWkAclr()
    eqs_Estop = 0: eqs_ixEstop = 0
    eqs_dEpk = 0: eqs_ixdEpk = 0: eqs_EdEpk = 0: eqs_ErrdEfpk = 0
    eqs_dEu2max = 0: eqs_ixdEu2max = 0: eqs_EdEu2max = 0: eqs_dEt2max = 0
    eqs_dEuG = 0: eqs_ixdEuG = 0: eqs_EdEuG = 0: eqs_ErrdE2ixE = 0

    eqsi_E0 = 0: eqsi_ixE0 = 0: eqsi_dE0 = 0
    eqsi_E1 = 0: eqsi_ixE1 = 0: eqsi_dE1 = 0
    eqsi_Slp = 0: eqsi_Edf = 0: eqsi_dEdf = 0
End Sub

'/*----------------------------------------------------------------------*/
'/*(Private Func.)  eq_* Work Area Dump as Debug.Print
'/*----------------------------------------------------------------------*/
Private Sub sr_eqWkAdmp()
    Debug.Print "eq_A1="; eq_A1; " _Th1="; eq_Th1; " _SwEt="; eq_SwEt
    Debug.Print "  _E10="; eq_E10; "@_ixE10="; eq_ixE10; " _R10="; eq_R10
    Debug.Print "  _E11="; eq_E11; "@_ixE11="; eq_ixE11; " _R11="; eq_R11; " _Et11="; eq_Et11
    Debug.Print "  _dEu1="; eq_dEu1; " _dEt1="; eq_dEt1
    Debug.Print "  _A2="; eq_A2; " _Th2="; eq_Th2
    Debug.Print "  _E20="; eq_E20; "@_ixE20="; eq_ixE20; " _R20="; eq_R20
    Debug.Print "  _E21="; eq_E21; "@_ixE21="; eq_ixE21; " _R21="; eq_R21; " _Et21="; eq_Et21
    Debug.Print "  _dEu2="; eq_dEu2; " _dEt2="; eq_dEt2; " _rV="; eq_rV
End Sub

'/*----------------------------------------------------------------------*/
'/*(Private Func.)  eq_* Work Area Dump (2) as Debug.Print
'/*----------------------------------------------------------------------*/
Private Sub sr_eqsWkAdmp()
    Debug.Print "eqs_Estop="; eqs_Estop; " @ ix="; eqs_ixEstop
    Debug.Print "   _dEpk="; eqs_dEpk; " @ ix="; eqs_ixdEpk; " E="; eqs_EdEpk; " Err="; eqs_ErrdEfpk
    Debug.Print "   _dEu2max="; eqs_dEu2max; " @ ix="; eqs_ixdEu2max; " E="; eqs_EdEu2max
    Debug.Print "   _dEt2max="; eqs_dEt2max
    Debug.Print "   _dEuG="; eqs_dEuG; " @ ix="; eqs_ixdEuG; " E="; eqs_EdEuG; " Err="; eqs_ErrdE2ixE
    
    Debug.Print "eqsi_E0 ="; eqsi_E0; " @ ix="; eqsi_ixE0; " dE0="; eqsi_dE0
    Debug.Print "    _E1 ="; eqsi_E1; " @ ix="; eqsi_ixE1; " dE1="; eqsi_dE1
    Debug.Print "    _Edf="; eqsi_Edf; " dEdf="; eqsi_dEdf; " _Slp="; eqsi_Slp
    Debug.Print "    _EG ="; eqsi_EG; " dEG ="; eqsi_dEG
End Sub

'/*----------------------------------------------------------------------*/
'/*(Func.)  Range equivalent Energy
'/* !WS1; Bm1&Tg1 : !Eu10 -> R10       [um]
'/* !WS2; Bm2&Tg2 : ?Eu20 -> R20 = R10 [um]
'/*  return: Eu20 where R10 = R20
'/*----------------------------------------------------------------------*/
Public Function srRng_eq_Eold(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double) As Variant
Attribute srRng_eq_Eold.VB_Description = "̊֐́ARng [m] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srRng_eq_Eold.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rT As Double, rV As Variant
    If (Eu10 <= 0) Then GoTo Err1
    sr_SetWS (WS1): sr_eqWkAclr: eq_E10 = Eu10
    If (Eu10 > Emax) Then GoTo Err1
    eq_R10 = sr_E2Rng(eq_E10): eq_ixE10 = ts_ixE
    If (ts_errE) Then GoTo Err8

    sr_SetWS (WS2)
    eq_E20 = sr_Rng2E(eq_R10)                    '<--[ Rng equiv. Eold ]
    eq_ixE20 = ts_ixE
    If (ts_errV) Then GoTo Err9
    eq_rV = eq_E20
    rV = eq_rV: GoTo ExF                         ' rV=       when
Err1:             rV = CVErr(xlErrNum): GoTo ExF ' = #NUM!   Eu10<=0 | Eu10>Emax
Err2: eq_rV = -1: rV = CVErr(xlErrNA): GoTo ExF  ' = #N/A
Err8: eq_rV = -8: rV = CVErr(xlErrNA): GoTo ExF  ' WS1 out of Rng(E) table [never as already checked]
Err9: eq_rV = -9: rV = CVErr(xlErrNA): GoTo ExF  ' WS2 out of Rng(E) table
ExF:  srRng_eq_Eold = rV
'Debug.Print "srRng_eq_Eold(Eu10="; Eu10; ")="; rV
'Call sr_eqWkAdmp
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Enew[MeV/u] equivalent Thikness     if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10      -> !Th1 -> Eu11
'/* !WS2; Bm2(A2)&Tg2 :  Eu20=Eu10 -> ?Th2 -> Eu21=E11
'/*  return: Th2 where Eu11 = Eu21
'/*----------------------------------------------------------------------*/
'/*(Func.)  Enewt[MeV] equivalent Thikness      if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10      -> !Th1 -> Eu11 ; Et11=Eu11*A1
'/* !WS2; Bm2(A2)&Tg2 :  Eu20=Eu10 -> ?Th2 -> Eu21 ; Et21=Eu21*A2=Et11
'/*  return: Th2 where Et11 = Et21
'/*----------------------------------------------------------------------*/
Private Function sr__Enew_eq_Th(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double, _
    ByVal Kg1 As Double, ByVal Kg2 As Double, _
    ByVal SwEt As Boolean) As Double
    
    sr_eqWkAclr
    eq_A1 = srInfoIonA(WS1): eq_E10 = Eu10: eq_SwEt = SwEt
    eq_Th1 = Th1 * Kg1
    If (Eu10 > Emax) Then GoTo Err11
    eq_E11 = sr_E2Enew(eq_E10, eq_Th1)
    eq_ixE10 = en_ixE0: eq_R10 = en_R0
    eq_ixE11 = en_ixE1: eq_R11 = en_R1: eq_dEu1 = en_dE
    If (ts_errE) Then GoTo Err18
    If ((Not SwEt) And (eq_E11 <= 0)) Then GoTo Err12
    eq_dEt1 = eq_dEu1 * eq_A1: eq_Et11 = eq_E11 * eq_A1
    
    eq_A2 = srInfoIonA(WS2)
    eq_E20 = eq_E10                         '<--[ precondition : E20 = E10 ]
    eq_R20 = sr_E2Rng(eq_E20): eq_ixE20 = ts_ixE
    If (ts_errE) Then GoTo Err19
    If (SwEt) Then
        If (eq_Et11 > (eq_E20 * eq_A2)) Then GoTo Err13
        eq_Et21 = eq_Et11                   '<--[ Enewt equiv. ]
        eq_E21 = eq_Et21 / eq_A2
    Else
        eq_E21 = eq_E11                     '<--[ Enew equiv. ]
        eq_Et21 = eq_E21 * eq_A2
    End If
    If (eq_E21 = 0) Then                    ' == eq_Et11=0 : just stopped
        eq_R21 = 0: eq_ixE21 = 0
    Else
        eq_R21 = sr_E2Rng(eq_E21): eq_ixE21 = ts_ixE
        If (ts_errE) Then GoTo Err19
    End If
    eq_Th2 = (eq_R20 - eq_R21) / Kg2        '<--[ Thick ]
    If (eq_Th2 < 0) Then GoTo Err14
    eq_rV = eq_Th2
    GoTo ExF
Err11: eq_rV = -1: GoTo ExF ' Eu10 > Emax
Err12: eq_rV = -2: GoTo ExF ' Eu11 < 0      as Th2 becomes indefinite
Err13: eq_rV = -3: GoTo ExF ' Et20 > Et11
Err14: eq_rV = -4: GoTo ExF ' R20 < R21
Err18: eq_rV = -8: GoTo ExF ' WS1 out of Rng(E) table  (never as already checked)
Err19: eq_rV = -9: GoTo ExF ' WS2 out of Rng(E) table
ExF:  sr__Enew_eq_Th = eq_rV
'Debug.Print "sr__Enew_eq_Th(Eu10="; Eu10; " Th1="; Th1; " Kg1,2="; Kg1; Kg2; ")="; eq_rV
'Call sr_eqWkAdmp
End Function

Public Function srEnew_eq_Th(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double) As Variant
Attribute srEnew_eq_Th.VB_Description = "̊֐́AEnew [MeV/u] ɂȂ Trg2 [m]@Ԃ܂B"
Attribute srEnew_eq_Th.VB_ProcData.VB_Invoke_Func = " \n20"
    
Dim rT As Double, rV As Variant
    If ((Eu10 <= 0) Or (Th1 < 0)) Then GoTo Err1
    rT = sr__Enew_eq_Th(WS1, WS2, Eu10, Th1, 1, 1, False)
    If (rT < 0) Then GoTo Err2
    rV = rT: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Th()
ExF:  srEnew_eq_Th = rV
End Function

Public Function srEnew_eq_ThGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srEnew_eq_ThGas.VB_Description = "̊֐́AEnew [MeV/u] ɂȂ GasTrg2 [mm]@Ԃ܂B"
Attribute srEnew_eq_ThGas.VB_ProcData.VB_Invoke_Func = " \n20"
    
Dim rT As Double, rV As Variant, Kg1 As Double, Kg2 As Double
    If (Not sr_Gas_eq_Gas(WS1, WS2, Pa1, degC1, Kg1, Pa2, degC2, Kg2)) Then GoTo Err1
    If ((Eu10 <= 0) Or (Tmm1 < 0)) Then GoTo Err1
    rT = sr__Enew_eq_Th(WS1, WS2, Eu10, Tmm1, Kg1, Kg2, False)
    If (rT < 0) Then GoTo Err2
    rV = rT: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0 | NotGas
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Th()
ExF:  srEnew_eq_ThGas = rV
End Function

Public Function srEnewt_eq_Th(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double) As Variant
Attribute srEnewt_eq_Th.VB_Description = "̊֐́AEnewt [MeV] ɂȂ Trg2 [m]@Ԃ܂B"
Attribute srEnewt_eq_Th.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rT As Double, rV As Variant
    If ((Eu10 <= 0) Or (Th1 < 0)) Then GoTo Err1
    rT = sr__Enew_eq_Th(WS1, WS2, Eu10, Th1, 1, 1, True)
    If (rT < 0) Then GoTo Err2
    rV = rT: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Th()
ExF:  srEnewt_eq_Th = rV
End Function

Public Function srEnewt_eq_ThGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srEnewt_eq_ThGas.VB_Description = "̊֐́AEnewt [MeV] ɂȂ GasTrg2 [mm]@Ԃ܂B"
Attribute srEnewt_eq_ThGas.VB_ProcData.VB_Invoke_Func = " \n20"
    
Dim rT As Double, rV As Variant, Kg1 As Double, Kg2 As Double
    If (Not sr_Gas_eq_Gas(WS1, WS2, Pa1, degC1, Kg1, Pa2, degC2, Kg2)) Then GoTo Err1
    If ((Eu10 <= 0) Or (Tmm1 < 0)) Then GoTo Err1
    rT = sr__Enew_eq_Th(WS1, WS2, Eu10, Tmm1, Kg1, Kg2, True)
    If (rT < 0) Then GoTo Err2
    rV = rT: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Th()
ExF:  srEnewt_eq_ThGas = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  Enew[MeV/u] equivalent Eold [MeV/u] if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10 -> !Th1     -> Eu11
'/* !WS2; Bm2(A2)&Tg2 : ?Eu20 ->  Th2=Th1 -> Eu21=E11
'/*  return: Eu20 where Eu11 = Eu21
'/*----------------------------------------------------------------------*/
'/*(Func.)  Enewt[MeV] equivalent Eold [MeV/u]  if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10 -> !Th1     -> Eu11 ; Et11=Eu11*A1
'/* !WS2; Bm2(A2)&Tg2 : ?Eu20 ->  Th2=Th1 -> Eu21 ; Et21=Eu21*A2=Et11
'/*  return: Eu20 where Et11 = Et21
'/*----------------------------------------------------------------------*/
Private Function sr__Enew_eq_Eold(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double, _
    ByVal Kg1 As Double, ByVal Kg2 As Double, _
    ByVal SwEt As Boolean) As Double
    
    sr_eqWkAclr
    eq_A1 = srInfoIonA(WS1): eq_E10 = Eu10: eq_SwEt = SwEt
    eq_Th1 = Th1 * Kg1
    If (Eu10 > Emax) Then GoTo Err11
    eq_E11 = sr_E2Enew(eq_E10, eq_Th1)
    eq_ixE10 = en_ixE0: eq_R10 = en_R0
    eq_ixE11 = en_ixE1: eq_R11 = en_R1: eq_dEu1 = en_dE
    If (ts_errE) Then GoTo Err18
    If ((Not SwEt) And (eq_E11 <= 0)) Then GoTo Err12
    eq_dEt1 = eq_dEu1 * eq_A1: eq_Et11 = eq_E11 * eq_A1
    
    eq_A2 = srInfoIonA(WS2)
    eq_Th2 = Th1 * Kg2                      '<--[ precondition : Th2 = Th1 ]
    If (SwEt) Then
        eq_Et21 = eq_Et11                   '<--[ Enewt equiv. ]
        eq_E21 = eq_Et21 / eq_A2
    Else
        eq_E21 = eq_E11                     '<--[ Enew equiv. ]
        eq_Et21 = eq_E21 * eq_A2
    End If
    If (eq_E21 = 0) Then                    ' == eq_Et11=0 : just stopped
        eq_E20 = sr_Rng2E(eq_Th2)           '<--[ Eold = Rng2E(Th2) ]
        eq_ixE20 = ts_ixE: eq_R20 = eq_Th2
        eq_ixE21 = 0: eq_R21 = 0
    Else
        eq_E20 = sr_E2Eold(eq_E21, eq_Th2)  '<--[ Eold ]
        eq_ixE20 = en_ixE0: eq_R20 = en_R0
        eq_ixE21 = en_ixE1: eq_R21 = en_R1
        If (ts_errE Or (eq_E20 <= 0)) Then GoTo Err19
    End If
    eq_rV = eq_E20
    GoTo ExF
Err11: eq_rV = -1: GoTo ExF ' Eu10 > Emax
Err12: eq_rV = -2: GoTo ExF ' Eu11 < 0      as Th2 becomes indefinite
Err18: eq_rV = -8: GoTo ExF ' WS1 out of Rng(E) table  (never as already checked)
Err19: eq_rV = -9: GoTo ExF ' WS2 out of Rng(E) table
ExF:  sr__Enew_eq_Eold = eq_rV
'Debug.Print "sr__Enew_eq_Eold(Eu10="; Eu10; " Th1="; Th1; " Kg1,2="; Kg1; Kg2; ")="; eq_rV
'Call sr_eqWkAdmp
End Function

Public Function srEnew_eq_Eold(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double) As Variant
Attribute srEnew_eq_Eold.VB_Description = "̊֐́AEnew [MeV/u] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srEnew_eq_Eold.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant
    If ((Eu10 <= 0) Or (Th1 < 0)) Then GoTo Err1
    rE = sr__Enew_eq_Eold(WS1, WS2, Eu10, Th1, 1, 1, False)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Eold()
ExF:  srEnew_eq_Eold = rV
End Function

Public Function srEnew_eq_EoldGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srEnew_eq_EoldGas.VB_Description = "̊֐́AEnew [MeV/u] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srEnew_eq_EoldGas.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant, Kg1 As Double, Kg2 As Double
    If (Not sr_Gas_eq_Gas(WS1, WS2, Pa1, degC1, Kg1, Pa2, degC2, Kg2)) Then GoTo Err1
    If ((Eu10 <= 0) Or (Tmm1 < 0)) Then GoTo Err1
    rE = sr__Enew_eq_Eold(WS1, WS2, Eu10, Tmm1, Kg1, Kg2, False)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0 | NotGas
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Eold()
ExF:  srEnew_eq_EoldGas = rV
End Function

Public Function srEnewt_eq_Eold(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double) As Variant
Attribute srEnewt_eq_Eold.VB_Description = "̊֐́AEnewt [MeV] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srEnewt_eq_Eold.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant
    If ((Eu10 <= 0) Or (Th1 < 0)) Then GoTo Err1
    rE = sr__Enew_eq_Eold(WS1, WS2, Eu10, Th1, 1, 1, True)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Eold()
ExF:  srEnewt_eq_Eold = rV
End Function

Public Function srEnewt_eq_EoldGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srEnewt_eq_EoldGas.VB_Description = "̊֐́AEnewt [MeV] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srEnewt_eq_EoldGas.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant, Kg1 As Double, Kg2 As Double
    If (Not sr_Gas_eq_Gas(WS1, WS2, Pa1, degC1, Kg1, Pa2, degC2, Kg2)) Then GoTo Err1
    If ((Eu10 <= 0) Or (Tmm1 < 0)) Then GoTo Err1
    rE = sr__Enew_eq_Eold(WS1, WS2, Eu10, Tmm1, Kg1, Kg2, True)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__Enew_eq_Eold()
ExF:  srEnewt_eq_EoldGas = rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  dEu[MeV/u] equivalent Thikness      if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10      -> !Th1 -> Eu11 ; dEu1=(Eu10-Eu11)
'/* !WS2; Bm2(A2)&Tg2 :  Eu20=Eu10 -> ?Th2 -> Eu21 ; dEu2=(Eu20-Eu21)=dEu1
'/*  return: Th2 where dEu1 = dEu2
'/*  note:   as Eu20=Eu10, dEu1=(Eu10-Eu11)=dEu2=(Eu10-Eu21)
'/*          then Eu21=Eu11, so that this func. is identical to srEnew_eq_Th()
'/*----------------------------------------------------------------------*/
Public Function srDEu_eq_Th(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double) As Variant
Attribute srDEu_eq_Th.VB_Description = "̊֐́AdEu [MeV/u] ɂȂ Trg2 [m]@Ԃ܂B"
Attribute srDEu_eq_Th.VB_ProcData.VB_Invoke_Func = " \n20"
    
    srDEu_eq_Th = srEnew_eq_Th(WS1, WS2, Eu10, Th1)
End Function

Public Function srDEu_eq_ThGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srDEu_eq_ThGas.VB_Description = "̊֐́AdEu [MeV/u] ɂȂ GasTrg2 [mm]@Ԃ܂B"
Attribute srDEu_eq_ThGas.VB_ProcData.VB_Invoke_Func = " \n20"
    
    srDEu_eq_ThGas = srEnew_eq_ThGas(WS1, WS2, Eu10, Tmm1, Pa1, degC1, Pa2, degC2)
End Function

'/*----------------------------------------------------------------------*/
'/*(Func.)  dEt[MeV] equivalent Thickness       if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10      -> !Th1 -> Eu11 ; dEt1=(Eu10-Eu11)*A1
'/* !WS2; Bm2(A2)&Tg2 :  Eu20=Eu10 -> ?Th2 -> Eu21 ; dEt2=(Eu20-Eu21)*A2=dEt1
'/*  return: Th2 where dEt1 = dEt2
'/*----------------------------------------------------------------------*/
Private Function sr__DEt_eq_Th(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double, _
    ByVal Kg1 As Double, ByVal Kg2 As Double, _
    ByVal SwEt As Boolean) As Double

    sr_eqWkAclr
    eq_A1 = srInfoIonA(WS1): eq_E10 = Eu10: eq_SwEt = SwEt
    eq_Th1 = Th1 * Kg1
    If (Eu10 > Emax) Then GoTo Err11
    eq_E11 = sr_E2Enew(eq_E10, eq_Th1)
    eq_ixE10 = en_ixE0: eq_R10 = en_R0
    eq_ixE11 = en_ixE1: eq_R11 = en_R1: eq_dEu1 = en_dE
    If (ts_errE) Then GoTo Err18
'#  If ((Not SwEt) And (eq_E11 <= 0)) Then GoTo Err12
    eq_dEt1 = eq_dEu1 * eq_A1: eq_Et11 = eq_E11 * eq_A1
   
    eq_A2 = srInfoIonA(WS2)
    eq_E20 = eq_E10                         '<--[ precondition : E20 = E10 ]
    eq_R20 = sr_E2Rng(eq_E20): eq_ixE20 = ts_ixE
    If (ts_errE) Then GoTo Err19
       
    eq_dEt2 = eq_dEt1                       '<--[ dEt equiv. ]
    eq_dEu2 = eq_dEt2 / eq_A2
    eq_E21 = eq_E20 - eq_dEu2
    If (eq_E21 < 0) Then GoTo Err15
    eq_R20 = sr_E2Rng(eq_E20): eq_ixE20 = ts_ixE
    If (ts_errE) Then GoTo Err19
    eq_R21 = sr_E2Rng(eq_E21): eq_ixE21 = ts_ixE
    If (ts_errE) Then GoTo Err19
    eq_Th2 = (eq_R20 - eq_R21) / Kg2        '<--[ Thick. ]
    If (eq_Th2 < 0) Then GoTo Err14
    eq_rV = eq_Th2
    GoTo ExF
Err11:  eq_rV = -1: GoTo ExF ' Eu10 > Emax
'Err12: eq_rV = -2: GoTo ExF ' Eu11 < 0      as Th2 becomes indefinite
Err14:  eq_rV = -4: GoTo ExF ' R20 < R21
Err15:  eq_rV = -5: GoTo ExF ' Eu10 < dEu2 ie dEt1 > Et20
Err18:  eq_rV = -8: GoTo ExF ' WS1 out of Rng(E) table  (never as already checked)
Err19:  eq_rV = -9: GoTo ExF ' WS2 out of Rng(E) table
ExF:  sr__DEt_eq_Th = eq_rV
'Debug.Print "sr__DEt_eq_Th(Eu10="; Eu10; " Th1="; Th1; " Kg1,2="; Kg1; Kg2; ")="; eq_rV
'Call sr_eqWkAdmp
End Function

Public Function srDEt_eq_Th(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double) As Variant
Attribute srDEt_eq_Th.VB_Description = "̊֐́AdEt [MeV] ɂȂ Trg2 [m]@Ԃ܂B"
Attribute srDEt_eq_Th.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rT As Double, rV As Variant
    If ((Eu10 <= 0) Or (Th1 < 0)) Then GoTo Err1
    rT = sr__DEt_eq_Th(WS1, WS2, Eu10, Th1, 1, 1, True)
    If (rT < 0) Then GoTo Err2
    rV = rT: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__DEt_eq_Th()
ExF:  srDEt_eq_Th = rV
End Function

Public Function srDEt_eq_ThGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srDEt_eq_ThGas.VB_Description = "̊֐́AdEt [MeV] ɂȂ GasTrg2 [mm]@Ԃ܂B"
Attribute srDEt_eq_ThGas.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rT As Double, rV As Variant, Kg1 As Double, Kg2 As Double
    If (Not sr_Gas_eq_Gas(WS1, WS2, Pa1, degC1, Kg1, Pa2, degC2, Kg2)) Then GoTo Err1
    If ((Eu10 <= 0) Or (Tmm1 < 0)) Then GoTo Err1
    rT = sr__DEt_eq_Th(WS1, WS2, Eu10, Tmm1, Kg1, Kg2, True)
    If (rT < 0) Then GoTo Err2
    rV = rT: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__DEt_eq_Th()
ExF:  srDEt_eq_ThGas = rV
End Function

'       ====================================================
'           Combination Func(3) *_eq_*() which need E-Rng table search
'       ====================================================

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.) Find dEu peak
'/*         ES     search start Eu -> return found peak E
'/*         Th      thick
'/*         Odr     search direction 1/-1 = Forward/Backward
'/*----------------------------------------------------------------------*/
Private Function sr_dEfpk(ByVal ES As Double, ByVal Th As Double, _
    ByVal Odr As Integer) As Boolean

Dim ixE As Integer, ixE0 As Integer, ixE1 As Integer, ixEs As Integer
Dim E0 As Double, E1 As Double, dE As Double
Dim dEpk As Double, E0pk As Double, rV As Boolean

    If ((ES < Emin) Or (ES > Emax)) Then GoTo Err1
    If (Th < 0) Then GoTo Err2
    
    ' search ixE range/direction
    ixE0 = sr_E2Ix(ES)
    ixE1 = IIf((Odr = 1), IxEmax, IxEmin)
    ixEs = IIf((Odr = 1), 1, -1)
    dEpk = -1
    For ixE = ixE0 To ixE1 Step ixEs
        E1 = sr_ixE2Enew(ixE, Th)   ' does Not care E1=-1 (stop in Th)
        E0 = en_E0: dE = en_dE
        If (dE < dEpk) Then GoTo Found
        dEpk = dE: E0pk = E0
    Next ixE
    GoTo Err3
Err1: eqs_ErrdEfpk = -1: rV = False: GoTo ExF ' ES is out of E table
Err2: eqs_ErrdEfpk = -2: rV = False: GoTo ExF ' Th < 0
Err3: eqs_ErrdEfpk = -3: rV = False: GoTo ExF ' not found (never happen usually)
Found: rV = True
      eqs_dEpk = dEpk: eqs_ixdEpk = ixE - 1: eqs_EdEpk = E0pk
ExF:  sr_dEfpk = rV
'Debug.Print "sr_dEfpk(ES="; ES; " Th="; Th; " Odr="; Odr; ")="; rV
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.) dEu search
'/*             Rng(ixE) Table search calculating dEu for each ixE
'/*         ES      search start Eu
'/*         Th      thick
'/*         dEG     goal dE value [MeV/u]
'/*         Ehl     E high/low side
'/*----------------------------------------------------------------------*/
Private Function sr_dE2ixE(ByVal ES As Double, ByVal Th As Double, _
    ByVal dEG As Double, ByVal Ehl As Integer) As Integer

Dim ixE As Integer, ixE0 As Integer, ixE1 As Integer, ixEs As Integer
Dim E0 As Double, E1 As Double, dE As Double, rV As Integer

    If ((ES < Emin) Or (ES >= Emax)) Then GoTo Err1
    If (Th < 0) Then GoTo Err2
    
    ' search ixE range/direction
    ixE0 = IIf((Ehl = 1), IxEmax, IxEmin)
    ixE1 = sr_E2Ix(ES)
    ixEs = IIf((Ehl = 1), -1, 1)
    For ixE = ixE0 To ixE1 Step ixEs
        E1 = sr_ixE2Enew(ixE, Th)   ' does Not care E1=-1 (stop in Th)
        E0 = en_E0: dE = en_dE
        If (dE >= dEG) Then GoTo Found
    Next ixE
    GoTo Err3
Err1: eqs_ErrdE2ixE = -1: rV = 0: GoTo ExF  ' ES is out of E table
Err2: eqs_ErrdE2ixE = -2: rV = 0: GoTo ExF  ' Th < 0
Err3: eqs_ErrdE2ixE = -3: rV = 0: GoTo ExF  ' not found (never happen usually)
Found: rV = ixE
      eqs_ixdEuG = ixE: eqs_EdEuG = E0
ExF:   sr_dE2ixE = rV
'Debug.Print "sr_dE2ixE(ES="; ES; " Th="; Th; " dEG="; dEG; " Ehl="; Ehl; ")="; rV
'Debug.Print "E0,E1,dE="; E0; E1; dE; "@ ixE="; ixE
End Function

'/*----------------------------------------------------------------------*/
'/*(Priv.Func.) E vs dE interpolation
'/*             calc. Linear interpolation  between ixE0 and ixE1
'/*     Th          thick of target [um]
'/*     dEG         goal  dE value [MeV/u]
'/*     ixE0,ixE1   neighboring two lines in the E,Rng(ixE) table
'/*----------------------------------------------------------------------*/
Private Function sr_dEip(ByVal Th As Double, ByVal dEG As Double, _
    ByVal ixE0 As Integer, ByVal ixE1 As Integer) As Double

Dim E As Double
    eqsi_ixE0 = ixE0: eqsi_ixE1 = ixE1: eqsi_dEG = dEG
    E = sr_ixE2Enew(eqsi_ixE0, Th): eqsi_E0 = en_E0: eqsi_dE0 = en_dE
    E = sr_ixE2Enew(eqsi_ixE1, Th): eqsi_E1 = en_E0: eqsi_dE1 = en_dE
    eqsi_Edf = eqsi_E1 - eqsi_E0
    eqsi_dEdf = eqsi_dE1 - eqsi_dE0
    eqsi_Slp = eqsi_dEdf / eqsi_Edf
    
    ' interpolation between E vs dE : as dEG = dE0 + slp * (E - E0) then
    eqsi_EG = (eqsi_dEG - eqsi_dE0) / eqsi_Slp + eqsi_E0
    sr_dEip = eqsi_EG
End Function

'/*---------------- for DBG use olny ------------------------------------*/
#If DBG Then

Public Function srdEfpk(ByVal WS As String, _
    ByVal ES As Double, ByVal Th As Double, _
    ByVal Odr As Integer) As Double

Dim st As Boolean
    sr_SetWS (WS)
    st = sr_dEfpk(ES, Th, Odr)
    srdEfpk = eqs_dEpk
End Function

Public Function srdEfpkE(ByVal WS As String, _
    ByVal ES As Double, ByVal Th As Double, _
    ByVal Odr As Integer) As Double
Dim dE As Double
    sr_SetWS (WS)
    dE = sr_dEfpk(ES, Th, Odr)
    srdEfpkE = eqs_EdEpk
End Function

Public Function srdE2ixE(ByVal WS As String, _
    ByVal ES As Double, ByVal Th As Double, _
    ByVal dEG As Double, ByVal Ehl As Integer) As Integer

    sr_SetWS (WS)
    srdE2ixE = sr_dE2ixE(ES, Th, dEG, Ehl)
End Function

Public Function srdEip(ByVal WS As String, _
    ByVal Th As Double, ByVal dEG As Double, _
    ByVal ixE1 As Integer, ByVal ixE2 As Integer) As Double
    
    sr_SetWS (WS)
    srdEip = sr_dEip(Th, dEG, ixE1, ixE2)
End Function

#End If
'/*---------------- for DBG use olny ------------------------------------*/

'/*----------------------------------------------------------------------*/
'/*(Func.)  dEu[MeV/u] equivalent Energy        if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10 -> !Th1     -> Eu11 ; dEu1=(Eu10-Eu11)
'/* !WS2; Bm2(A2)&Tg2 : ?Eu20 ->  Th2=Th1 -> Eu21 ; dEu2=(Eu20-Eu21)=dEu1
'/*  return: Eu20 where dEu1 = dEu2
'/*----------------------------------------------------------------------*/
'/*(Func.)  dEt[MeV] equivalent Energy          if Gas, Th[um]->Th[mm]
'/* !WS1; Bm1(A1)&Tg1 : !Eu10 -> !Th1     -> Eu11 ; dEt1=(Eu10-Eu11)*A1
'/* !WS2; Bm2(A2)&Tg2 : ?Eu20 ->  Th2=Th1 -> Eu21 ; dEt2=(Eu20-Eu21)*A2=dEt1
'/*  return: Eu20 where dEt1 = dEt2
'/*----------------------------------------------------------------------*/
'/*(Priv.Func.) common for dE{u|t} equivalent Energy
'/*     Ehl=   +1/-1    table search from      ixEmax/ixEmin
'/*     dEut = +1/-1    equivalent search mode    dEu/dEt
'/*----------------------------------------------------------------------*/
Private Function sr__DE_eq_Eold(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double, _
    ByVal Ehl As Integer, ByVal dEut As Integer, _
    ByVal Kg1 As Double, ByVal Kg2 As Double) As Double

Dim ok As Boolean, ixE0 As Integer, ixE1 As Integer
    Call sr_eqWkAclr: Call sr_eqsWkAclr
    eq_A1 = srInfoIonA(WS1): eq_E10 = Eu10
    eq_SwEt = IIf((dEut = 1), False, True)
    eq_Th1 = Th1 * Kg1
    If (Eu10 > Emax) Then GoTo Err11
    eq_E11 = sr_E2Enew(eq_E10, eq_Th1)
    eq_ixE10 = en_ixE0: eq_R10 = en_R0
    eq_ixE11 = en_ixE1: eq_R11 = en_R1: eq_dEu1 = en_dE
    If (ts_errE) Then GoTo Err18
    eq_dEt1 = eq_dEu1 * eq_A1: eq_Et11 = eq_E11 * eq_A1
    
    eq_A2 = srInfoIonA(WS2)
    eq_Th2 = Th1 * Kg2                      '<--[ precondition : Th2 = Th1 ]
    
    '   dEu peak search from E20 stop energy
    eqs_Estop = sr_Rng2E(eq_Th2)            ' Eu20 just stop after Th2=Th1
    eqs_ixEstop = ts_ixE
    ok = sr_dEfpk(Emax, eq_Th2, -1)         ' dEu peak search from Emax
    If (Not ok) Then GoTo Err21             '  (return eqs_dEpk, _ixdEpk, _EdEpk)
    
    If (eqs_dEpk > eqs_Estop) Then          ' dEu2max = dEu peak
        eqs_dEu2max = eqs_dEpk
        eqs_ixdEu2max = eqs_ixdEpk: eqs_EdEu2max = eqs_EdEpk
    Else                                    ' dEu2max = Eu20 stop
        eqs_dEu2max = eqs_Estop             '  at Estop, dEu2=Estop
        eqs_ixdEu2max = eqs_ixEstop: eqs_EdEu2max = eqs_Estop
    End If
    eqs_dEt2max = eqs_dEu2max * eq_A2
    
    '   dE goal val(dEuG) search & get its ixE
    If (dEut = 1) Then  ' dEu search mode
        If (eqs_dEu2max < eq_dEu1) Then GoTo Err22   ' no answer
        If (eqs_dEu2max = eq_dEu1) Then GoTo Err23   ' only one answer
        eqs_dEuG = eq_dEu1                  '<--[ dEu equivalent ]
    Else                ' dEt search mode
        If (eqs_dEt2max < eq_dEt1) Then GoTo Err22   ' no answer
        If (eqs_dEt2max = eq_dEt1) Then GoTo Err23   ' only one answer
        eqs_dEuG = eq_dEt1 / eq_A2          '<--[ dEt equivalent ]
    End If
                                            '<--[ Eold (dEuG) search ]
    ixE1 = sr_dE2ixE(eqs_EdEu2max, eq_Th2, eqs_dEuG, Ehl)
    If (ixE1 <= 0) Then GoTo Err24
        
    '   interpolation between E vs dE
    ixE0 = IIf((Ehl > 0), ixE1 + 1, ixE1 - 1)
    eq_ixE20 = ixE0
    If ((ixE0 < IxEmin) Or (ixE0 > IxEmax)) Then GoTo Err24
    If ((ixE1 < IxEmin) Or (ixE1 > IxEmax)) Then GoTo Err24
                                           '<--[ Eold interpolation ]
    eq_E20 = sr_dEip(eq_Th2, eqs_dEuG, ixE0, ixE1)
    eq_rV = eq_E20
    GoTo ExF
Err11:  eq_rV = -1:  GoTo ExF ' Eu10 > Emax
Err18:  eq_rV = -8:  GoTo ExF ' WS1 out of Rng(E) table  (never as already checked)
Err21:  eq_rV = -11: GoTo ExF ' dE peak search err
Err22:  eq_rV = -12: GoTo ExF ' no answer
Err23:  eq_rV = eqs_EdEu2max: GoTo ExF ' only one answer
Err24:  eq_rV = -14: GoTo ExF ' dE2ixE err
ExF:  sr__DE_eq_Eold = eq_rV
'Debug.Print "sr__DEt_eq_Eold(Eu10="; Eu10; " Th1="; Th1; " Ehl,dEut="; Ehl; dEut; _
'    "Kg1,2="; Kg1; Kg2; ")="; eq_rV
'Call sr_eqWkAdmp: Call sr_eqsWkAdmp
End Function

Public Function srDEu_eq_Eold(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double, ByVal Ehl As Integer) As Variant
Attribute srDEu_eq_Eold.VB_Description = "̊֐́AdEu [MeV/u] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srDEu_eq_Eold.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant
    If ((Eu10 <= 0) Or (Th1 <= 0)) Then GoTo Err1
    rE = sr__DE_eq_Eold(WS1, WS2, Eu10, Th1, Ehl, 1, 1, 1)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__DE_eq_Eold()
ExF:  srDEu_eq_Eold = rV
End Function

Public Function srDEu_eq_EoldGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, ByVal Ehl As Integer, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srDEu_eq_EoldGas.VB_Description = "̊֐́AdEu [MeV/u] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srDEu_eq_EoldGas.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant, Kg1 As Double, Kg2 As Double
    If (Not sr_Gas_eq_Gas(WS1, WS2, Pa1, degC1, Kg1, Pa2, degC2, Kg2)) Then GoTo Err1
    If ((Eu10 <= 0) Or (Tmm1 < 0)) Then GoTo Err1
    rE = sr__DE_eq_Eold(WS1, WS2, Eu10, Tmm1, Ehl, 1, Kg1, Kg2)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__DEt_eq_Eold()
ExF:  srDEu_eq_EoldGas = rV
End Function

Public Function srDEt_eq_Eold(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Th1 As Double, ByVal Ehl As Integer) As Variant
Attribute srDEt_eq_Eold.VB_Description = "̊֐́AdEt [MeV] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srDEt_eq_Eold.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant
    If ((Eu10 <= 0) Or (Th1 <= 0)) Then GoTo Err1
    rE = sr__DE_eq_Eold(WS1, WS2, Eu10, Th1, Ehl, -1, 1, 1)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__DE_eq_Eold()
ExF:  srDEt_eq_Eold = rV
End Function

Public Function srDEt_eq_EoldGas(ByVal WS1 As String, ByVal WS2 As String, _
    ByVal Eu10 As Double, ByVal Tmm1 As Double, ByVal Ehl As Integer, _
    ByVal Pa1 As Double, ByVal degC1 As Double, _
    ByVal Pa2 As Double, ByVal degC2 As Double) As Variant
Attribute srDEt_eq_EoldGas.VB_Description = "̊֐́AdEt [MeV] ɂȂ Bm2 ʉߑOE [MeV/u] ; Eold@Ԃ܂B"
Attribute srDEt_eq_EoldGas.VB_ProcData.VB_Invoke_Func = " \n20"

Dim rE As Double, rV As Variant, Kg1 As Double, Kg2 As Double
    If (Not sr_Gas_eq_Gas(WS1, WS2, Pa1, degC1, Kg1, Pa2, degC2, Kg2)) Then GoTo Err1
    If ((Eu10 <= 0) Or (Tmm1 < 0)) Then GoTo Err1
    rE = sr__DE_eq_Eold(WS1, WS2, Eu10, Tmm1, Ehl, -1, Kg1, Kg2)
    If (rE < 0) Then GoTo Err2
    rV = rE: GoTo ExF                   ' rV=       when
Err1: rV = CVErr(xlErrNum): GoTo ExF    ' = #NUM!   Eu10<=0 | Th1 < 0
Err2: rV = CVErr(xlErrNA): GoTo ExF     ' = #N/A    see sr__DEt_eq_Eold()
ExF:  srDEt_eq_EoldGas = rV
End Function

'/*----------------------------------------------------------------------*/
'/*                         END of SRIMfit
'/*----------------------------------------------------------------------*/
